from django.shortcuts import render
from magplan.models import Section
from django.contrib.auth.decorators import login_required


# Create your views here.
@login_required
def index(request):
    return render(request, 'magplan/sections/index.html', {
        'sections': Section.objects.order_by('sort').all()
    })



from django.contrib.auth.decorators import login_required
from django.db.models import Q
from django.http import JsonResponse

from magplan.models import Issue, User


@login_required
def issues_search(request):
    q = request.GET.get('q', None)
    issues = Issue.objects.filter(number__exact=q)
    return JsonResponse(
        [{'id': issue.id, 'text': issue.__str__()} for issue in issues]
        , safe=False)


@login_required
def authors_search(request):
    q = request.GET.get('q', None)

    # FIXME: heavy request, should be optimized
    users = User.objects.filter(
        Q(email__icontains=q) |
        Q(meta__icontains=q) |
        Q(profile__f_name__icontains=q) |
        Q(profile__l_name__icontains=q) |
        Q(profile__m_name__icontains=q)
    )
    return JsonResponse(
        [{'id': user.id, 'text': user.__str__()} for user in users]
        , safe=False)



import datetime
import typing as tp

from django.contrib.auth.decorators import login_required
from django.db.models import Q, QuerySet
from django.shortcuts import render, redirect
from django.utils.timezone import now

from magplan.models import Post, Stage, User
from magplan.forms import WhitelistedPostExtendedModelForm, AdPostExtendedModelForm


def _get_filtered_posts_queryset(filter_: tp.Optional[str], current_user: User) -> QuerySet:
    """Returns articles QuerySet, based on user request filters.
    """
    posts = Post.objects \
        .prefetch_related('section', 'stage', 'issues__magazine', 'editor__profile') \
        .order_by('-updated_at')

    # Get filtered queryset
    if filter_ == 'self':
        posts = posts.filter(editor=current_user)
    elif filter_ == 'overdue':
        posts = posts \
            .filter(finished_at__lte=datetime.datetime.now()) \
            .exclude(stage__slug='vault') \
            .exclude(stage__slug='published')
    elif filter_ == 'vault':
        posts = posts.filter(stage__slug='vault')
    else:
        # Render recent by default
        datetime_now = now()
        filter_kwargs = {
            'updated_at__gte': datetime_now - datetime.timedelta(2 * 30),
            'updated_at__lte': datetime_now,
        }
        posts = posts.filter(**filter_kwargs)

    return posts


@login_required
def index(request):
    filter_ = request.GET.get('filter')
    posts: QuerySet = _get_filtered_posts_queryset(filter_, request.user)
    return render(request, 'magplan/articles/index.html', {
        'posts': posts,
        'filter_': filter_,
    })


@login_required
def whitelisted(request):
    form = WhitelistedPostExtendedModelForm()

    if request.method == 'POST':
        form = WhitelistedPostExtendedModelForm(request.POST)

        if form.is_valid():
            post = form.save(commit=False)
            post.editor = request.user
            post.stage = Stage.objects.get(slug='waiting')

            post.save()
            form.save_m2m()

            return redirect('posts_show', post.id)

    return render(request, 'magplan/articles/whitelisted.html', {
        'form': form,
    })


@login_required
def advert(request):
    form = AdPostExtendedModelForm()

    if request.method == 'POST':
        form = AdPostExtendedModelForm(request.POST)

        if form.is_valid():
            post = form.save(commit=False)
            post.editor = request.user
            post.stage = Stage.objects.get(slug='waiting')

            post.save()

            form.cleaned_data['issues'] = [form.cleaned_data['issues']]  # HACK: cast object as list to support m2m
            form.save_m2m()

            return redirect('posts_show', post.id)

    return render(request, 'magplan/articles/advert.html', {
        'form': form,
    })


@login_required
def search(request):
    if request.method == 'POST':
        q = request.POST['search_query']
        posts = (Post.objects
                 .prefetch_related('section', 'stage', 'issues__magazine', 'editor__profile')
                 .filter(Q(title__icontains=q) | Q(kicker__icontains=q)))

        return render(request, 'magplan/articles/index.html', {
            'posts': posts,
            'q': q,
        })



# Generated by Django 3.1.2 on 2020-11-21 17:50

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('magplan', '0002_auto_20201115_1140'),
    ]

    operations = [
        migrations.DeleteModel(
            name='Widget',
        ),
        migrations.DeleteModel(
            name='Widgetype',
        ),
        migrations.RemoveField(
            model_name='post',
            name='postype',
        ),
        migrations.DeleteModel(
            name='Postype',
        ),
    ]



from typing import Any, Callable


def safe_cast(value: Any, to: Callable, on_error: Any = None) -> Any:
    """"Safe casts value to type with provided casting constructor.

    If cast falls for some reason, default on_error value is returned.*

    :param value: Source value to cast*
    :param to: Callable constructor, which can accept typed value*
    :param on_error: Value to return is cast falls: bad constructor, unacceptable source type*
     :return: casted value or fallback*
    """
    try:
        casted: Any = to(value)
    except (ValueError, TypeError):  # noqa
        return on_error

    return casted



from django.contrib import messages
from django.contrib.auth.decorators import login_required
from django.shortcuts import redirect, render
from dynamic_preferences.users.forms import user_preference_form_builder


@login_required
def index(request):
    PreferencesForm = user_preference_form_builder(instance=request.user)
    if request.method == 'POST':
        form = PreferencesForm(request.POST)
        if form.is_valid():
            form.update_preferences()
            messages.add_message(
                request, messages.SUCCESS, "Настройи успешно обновлены"
            )

            return redirect('preferences_index')
        else:
            # TODO: check, if form reflects invalid
            #       payload for PreferencesForm
            PreferencesForm = user_preference_form_builder(instance=request.user)

    return render(request, 'magplan/preferences/index.html', {'form': PreferencesForm})



import datetime
import re
from typing import List

from django import template
from django.conf import settings
from django.urls import reverse
from django.utils.functional import SimpleLazyObject

from magplan.models import Vote, Comment
from magplan.utils import safe_cast
from magplan.views import posts, ideas

register = template.Library()

MAX_VOTE_OPTIONS_NUMBER = len(Vote.SCORE_CHOICES)
VOTE_SCORE_STEP = 100 // (MAX_VOTE_OPTIONS_NUMBER - 1)
FAILBACK_VOTE_INDEX = 2
CSS_COLORS = (
    'danger',
    'warning',
    'light',
    'info',
    'success',
)

EMOJI_SCORES = (
    '🤮',
    '🤨',
    '😐',
    '😏',
    '😍',
)


@register.filter(name='voted')
def voted(value, user):
    return value.voted(user)


@register.filter(name='humanize_score_index')
def humanize_score_index(index) -> str:
    if not isinstance(index, int):
        index = safe_cast(index, int, -1)

    if index == -1:  # error cast
        return Vote.SCORE_CHOICES[FAILBACK_VOTE_INDEX][1]

    score = index * VOTE_SCORE_STEP

    # Provide untrusted data
    # Underlying function will validate data
    return humanize_score(score)


@register.filter(name='humanize_score')
def humanize_score(value) -> str:
    if not isinstance(value, int):
        value = safe_cast(value, int, -1)

    if value == -1:  # error cast
        return Vote.SCORE_CHOICES[FAILBACK_VOTE_INDEX][1]

    if value % VOTE_SCORE_STEP != 0:
        return Vote.SCORE_CHOICES[FAILBACK_VOTE_INDEX][1]

    if not (0 <= value <= 100):
        return Vote.SCORE_CHOICES[FAILBACK_VOTE_INDEX][1]

    choice_index = value // VOTE_SCORE_STEP

    return Vote.SCORE_CHOICES[choice_index][1]


@register.filter(name='times')
def times(number):
    return range(number)


@register.filter(name='trim_filename')
def trim_filename(filename):
    if len(filename) > 30:
        return '%s...%s' % (
            filename[:10],
            filename[-20:]
        )
    return filename


@register.filter
def divide(value, arg):
    try:
        return int(value) / int(arg)
    except (ValueError, ZeroDivisionError):
        return None


class SetVarNode(template.Node):

    def __init__(self, var_name, var_value):
        self.var_name = var_name
        self.var_value = var_value

    def render(self, context):
        try:
            value = template.Variable(self.var_value).resolve(context)
        except template.VariableDoesNotExist:
            value = ""
        context[self.var_name] = value

        return u""


@register.tag(name='set')
def set_var(parser, token):
    """
    {% set some_var = '123' %}
    """
    parts = token.split_contents()
    if len(parts) < 4:
        raise template.TemplateSyntaxError(
            "'set' tag must be of the form: {% set <var_name> = <var_value> %}")

    return SetVarNode(parts[1], parts[3])


@register.filter
def can_be_moved_to_stage_by(post, user: SimpleLazyObject):
    return (post.stage.assignee and post.stage.assignee == user.user) or \
           (not post.stage.assignee and post.editor == user.user)


@register.filter
def sub(value, arg):
    return value - arg


@register.filter
def url_for_post_comments(post):
    return reverse(posts.comments, kwargs={
        'post_id': post.id,
    })


@register.filter
def url_for_idea_comments(idea):
    return reverse(ideas.comments, kwargs={
        'idea_id': idea.id,
    })


@register.filter
def is_overdue(date):
    return date > datetime.datetime.now()


# @register.filter
# def humazine_payment_type(payment):
#     return Payment.FORMAT_CHOICES[payment.format][1]


@register.filter
def count_human_comments(comments: List[Comment]) -> int:
    """Count comment from arrays, made by real users.

    It seems, plain iterator with incrementable counter may be faster
    as it does not produce additional map in closure.

        counter = 0
        for comment in comments:
            if comment.type != Comment.TYPE_SYSTEM:
                counter+=1
        return counter

    :param comments: Array of Comment instances
    :return: Actual comments number
    """
    return len(list(filter(lambda c: c.type != Comment.TYPE_SYSTEM, comments)))


@register.filter
def date_class(datetime_obj: datetime.datetime) -> str:
    """Return numeric representaion of provided dates, where:

    datetime_obj|f = 1 if passed
    datetime_obj|f = 2 if today
    datetime_obj|f = 0 else
    """
    # Convert 'datetime' => 'date' class for easier comparison
    date = datetime_obj.date()
    today = datetime.datetime.now().date()

    if date < today:
        return 'past'
    elif date == today:
        return 'today'
    else:
        return ''


@register.filter
def get_item(dictionary, key):
    return dictionary[key]


numeric_test = re.compile("^\d+$")


@register.filter(name='get_attr')
def get_attr(value, arg):
    """Gets an attribute of an object dynamically from a string name"""

    if hasattr(value, str(arg)):
        return getattr(value, arg)
    elif hasattr(value, 'has_key') and value.has_key(arg):
        return value[arg]
    elif numeric_test.match(str(arg)) and len(value) > int(arg):
        return value[int(arg)]
    else:
        return settings.TEMPLATE_STRING_IF_INVALID


@register.filter(name='score_css_color_by_index')
def score_css_color_by_index(index):
    if not isinstance(index, int):
        index = safe_cast(index, int, -1)

    if index == -1:  # error cast
        return CSS_COLORS[FAILBACK_VOTE_INDEX]

    if not (0 <= index <= (MAX_VOTE_OPTIONS_NUMBER - 1)):
        return CSS_COLORS[FAILBACK_VOTE_INDEX]

    return CSS_COLORS[index]


@register.filter(name='emoji_sign_by_index')
def emoji_sign_by_index(index):
    if not isinstance(index, int):
        index = safe_cast(index, int, -1)

    if index == -1:  # error cast
        return EMOJI_SCORES[FAILBACK_VOTE_INDEX]

    if not (0 <= index <= (MAX_VOTE_OPTIONS_NUMBER - 1)):
        return EMOJI_SCORES[FAILBACK_VOTE_INDEX]

    return EMOJI_SCORES[index]



import datetime
import logging
import mimetypes
import os
import typing as tp
from typing import List

import html2text
from django.contrib.auth import get_user_model
from django.contrib.contenttypes.fields import GenericRelation, GenericForeignKey
from django.contrib.contenttypes.models import ContentType
from django.core.mail import EmailMultiAlternatives
from django.db import models
from django.db.models import Q
from django.db.models.signals import post_save, pre_save
from django.template.loader import render_to_string
from django.utils import timezone

from magplan.conf import settings as  config
from magplan.integrations.images import S3Client
from magplan.integrations.posts import replace_images_paths, update_ext_db_xmd
from magplan.xmd import render_md
from magplan.xmd.mappers import s3_public_mapper as s3_image_mapper

NEW_IDEA_NOTIFICATION_PREFERENCE_NAME = 'magplan__new_idea_notification'

import enum
import hashlib

import django
import requests
from botocore.exceptions import ClientError
from django.contrib.postgres.fields import JSONField
from django.dispatch import receiver

from .xmd.mappers import plan_internal_mapper as plan_image_mapper

UserModel = get_user_model()


class StorageType(enum.Enum):
    S3 = 1


class AbstractBase(models.Model):
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)
    _old_id = models.PositiveIntegerField(null=True, blank=True)

    class Meta:
        abstract = True


class User(UserModel):
    class Meta:
        proxy = True

    meta = JSONField(default=dict)

    def __str__(self):
        p = self.profile
        if p.l_name and p.f_name:
            return '%s %s' % (p.l_name, p.f_name)
        elif p.n_name:
            return p.n_name
        else:
            return self.email

    @property
    def str_reverse(self):
        return self.__str__()

    @property
    def str_employee(self):
        return self.__str__()

    class Meta:
        permissions = (
            ("access_magplan", "Can access magplan"),
            ("manage_authors", "Can manage authors"),
        )

    def is_member(self, group_name: str) -> bool:
        """Check if user is member of group

        :param group_name: Group name to check user belongs to
        :return: True if a memeber, otherwise False
        """
        return self.groups.filter(name=group_name).exists()


class Profile(AbstractBase):
    is_public = models.BooleanField(null=False, blank=False, default=False)
    user = models.OneToOneField(User, on_delete=models.CASCADE, related_name='profile')
    f_name = models.CharField('Имя', max_length=255, blank=True, null=True)
    m_name = models.CharField('Отчество', max_length=255, blank=True, null=True)
    l_name = models.CharField('Фамилия', max_length=255, blank=True, null=True)
    n_name = models.CharField('Ник', max_length=255, blank=True, null=True)

    RUSSIA = 0
    UKRAINE = 1
    BELARUS = 2
    KAZAKHSTAN = 3
    COUNTRY_CHOICES = (
        (RUSSIA, 'Россия'),
        (UKRAINE, 'Украина'),
        (BELARUS, 'Беларусь'),
        (KAZAKHSTAN, 'Казахстан'),
    )
    country = models.SmallIntegerField(
        'Страна', choices=COUNTRY_CHOICES, default=RUSSIA
    )
    city = models.CharField('Город или поселок', max_length=255, blank=True, null=True)
    notes = models.TextField('Примечания', blank=True, null=True)


class Section(AbstractBase):
    def __str__(self):
        return self.title

    slug = models.SlugField(null=False, blank=False, max_length=255)
    title = models.CharField(null=False, blank=False, max_length=255)
    description = models.TextField(null=True, blank=False)
    sort = models.SmallIntegerField(null=False, blank=False, default=0)
    color = models.CharField(null=False, blank=False, default='000000', max_length=6)
    is_archived = models.BooleanField(null=False, blank=False, default=False)
    is_whitelisted = models.BooleanField(null=False, blank=False, default=False)


class Magazine(AbstractBase):
    slug = models.SlugField(null=False, blank=False, max_length=255)
    title = models.CharField(null=False, blank=False, max_length=255)
    description = models.TextField(null=False, blank=True)

    def __str__(self):
        return self.title


class Issue(AbstractBase):
    class Meta:
        ordering = ['-number']

    def __str__(self):
        return '%s #%s' % (self.magazine, self.number)

    number = models.SmallIntegerField(null=False, blank=False, default=0)
    title = models.CharField(null=True, blank=False, max_length=255)
    description = models.TextField(null=True, blank=False)
    magazine = models.ForeignKey(Magazine, on_delete=models.CASCADE)
    published_at = models.DateField(
        null=False, blank=False, default=datetime.date.today
    )

    @property
    def full_title(self) -> str:
        return '{} #{} {}'.format(
            'Хакер', self.number, self.title or ''
        )


class Stage(AbstractBase):
    def __str__(self):
        return self.title

    slug = models.SlugField(null=False, blank=False, max_length=255)
    title = models.CharField(null=False, blank=False, max_length=255)
    sort = models.SmallIntegerField(null=False, blank=False, default=0)
    duration = models.SmallIntegerField(null=True, blank=True, default=1)
    assignee = models.ForeignKey(User, null=True, blank=True, on_delete=models.CASCADE)
    prev_stage = models.ForeignKey(
        'self', related_name='n_stage', null=True, blank=True, on_delete=models.CASCADE
    )
    next_stage = models.ForeignKey(
        'self', related_name='p_stage', null=True, blank=True, on_delete=models.CASCADE
    )
    skip_notification = models.BooleanField(null=False, blank=False, default=False)
    meta = JSONField(default=dict)


class Idea(AbstractBase):
    AUTHOR_TYPE_NO = 'NO'
    AUTHOR_TYPE_NEW = 'NW'
    AUTHOR_TYPE_EXISTING = 'EX'
    AUTHOR_TYPE_CHOICES = [
        (AUTHOR_TYPE_NO, 'Нет автора'),
        (AUTHOR_TYPE_NEW, 'Новый автор'),
        (AUTHOR_TYPE_EXISTING, 'Существующий автор(ы)'),
    ]
    title = models.CharField(
        null=False, blank=False, max_length=255, verbose_name='Заголовок'
    )
    description = models.TextField(verbose_name='Описание')
    approved = models.BooleanField(null=True)
    editor = models.ForeignKey(User, on_delete=models.CASCADE, related_name='editor')
    post = models.OneToOneField(
        'Post', on_delete=models.SET_NULL, null=True, blank=True
    )
    comments = GenericRelation('Comment')
    author_type = models.CharField(
        max_length=2,
        choices=AUTHOR_TYPE_CHOICES,
        default=AUTHOR_TYPE_NO,
        verbose_name='Автор',
    )
    authors_new = models.CharField(
        max_length=255, null=True, blank=True, verbose_name='Новые автор'
    )
    authors = models.ManyToManyField(
        User, verbose_name='Авторы', related_name='authors', blank=True
    )

    def voted(self, user):
        vote = next((v for v in self.votes.all() if v.user_id == user.id), None)

        if vote:
            return True
        return False

    def _send_vote_notification(self, recipient: User) -> None:
        subject = f'Новая идея «{self.title}». Голосуйте!'

        context = {
            'idea': self,
            'APP_URL': os.environ.get('APP_URL')
        }
        message_html_content: str = render_to_string('email/new_idea.html', context)
        message_text_content: str = html2text.html2text(message_html_content)

        msg = EmailMultiAlternatives(subject, message_text_content, config.PLAN_EMAIL_FROM,
                                     [recipient.email])
        msg.attach_alternative(message_html_content, 'text/html')
        msg.send()

    def send_vote_notifications(self) -> None:
        active_users: tp.List[User] = User.objects.filter(is_active=True).exclude(id=self.editor_id)
        recipients: tp.List[User] = [
            user for user in active_users
            if user.preferences[NEW_IDEA_NOTIFICATION_PREFERENCE_NAME]
        ]

        for recipient in recipients:
            self._send_vote_notification(recipient)

    def __str__(self):
        return self.title

    class Meta:
        permissions = (
            ('edit_extended_idea_attrs', 'Edit extended Idea attributes'),
            ('recieve_idea_email_updates', 'Recieve email updates for Idea'),
        )

    @property
    def comments_(self):
        return self.comments.order_by('created_at').all

    @property
    def score(self):
        MAX_SCORE = 100

        all_scores = sum([v.score for v in self.votes.all()])
        max_scores = len(self.votes.all()) * MAX_SCORE

        return round(all_scores / max_scores * 100)

    @property
    def description_html(self):
        return render_md(self.description)


class Post(AbstractBase):
    # Used for external parser configuration
    PAYWALL_NOTICE_HEAD = '<div class="paywall-notice">'
    PAYWALL_NOTICE_BODY = 'Продолжение статьи доступно только продписчикам'
    PAYWALL_NOTICE_TAIL = '</div>'
    PAYWALL_NOTICE_RENDERED = '{}{}{}'.format(
        PAYWALL_NOTICE_HEAD, PAYWALL_NOTICE_BODY, PAYWALL_NOTICE_TAIL
    )

    def __str__(self):
        if self.kicker is None:
            return self.title

        separator = ' ' if self.kicker.endswith(('!', ':', '?')) else '. '
        return f'{self.kicker}{separator}{self.title}'

    POST_FORMAT_DEFAULT = 0
    POST_FORMAT_FEATURED = 1
    POST_FORMAT_CHOICES = (
        (POST_FORMAT_DEFAULT, 'Default'),
        (POST_FORMAT_FEATURED, 'Featured'),
    )

    format = models.SmallIntegerField(
        choices=POST_FORMAT_CHOICES, default=POST_FORMAT_DEFAULT
    )
    finished_at = models.DateTimeField(
        null=False,
        blank=False,
        default=django.utils.timezone.now,
        verbose_name='Дедлайн',
    )
    published_at = models.DateTimeField(
        null=True, blank=True, verbose_name='Дата публикации'
    )
    kicker = models.CharField(null=True, blank=True, max_length=255)
    slug = models.SlugField(null=True, blank=True, max_length=255)
    title = models.CharField(
        null=True, blank=True, max_length=255, verbose_name='Заголовок статьи'
    )
    description = models.TextField(
        null=False, blank=True, verbose_name='Описание статьи'
    )
    views = models.IntegerField(default=0)
    is_paywalled = models.BooleanField(default=False)
    xmd = models.TextField(null=True)
    html = models.TextField(null=True)

    editor = models.ForeignKey(
        User,
        null=True,
        on_delete=models.CASCADE,
        related_name="edited",
        verbose_name='Редактор',
    )
    authors = models.ManyToManyField(User, verbose_name='Авторы')
    stage = models.ForeignKey(Stage, on_delete=models.CASCADE, verbose_name='Этап')
    issues = models.ManyToManyField(Issue, related_name='posts', verbose_name='Выпуски')
    section = models.ForeignKey(
        Section,
        on_delete=models.CASCADE,
        null=False,
        blank=False,
        verbose_name='Раздел',
    )
    last_updater = models.ForeignKey(
        User,
        related_name='posts_updated',
        verbose_name='Кто последний обновлял',
        null=True,
        on_delete=models.SET_NULL,
    )

    meta = JSONField(default=dict)

    comments = GenericRelation('Comment')
    is_locked = models.BooleanField(default=False)
    css = models.TextField(
        null=True, blank=True, verbose_name='CSS-стили для статьи',
        help_text=(
            'CSS, который будет применяться к превью статьи'
        )
    )

    def imprint_updater(self, user: User) -> None:
        """Update updated_at timestamp and set provided user as last_updater.

        `created_at` may be overwritten further on DB-hook level.
        .save() method should be explicitly called.
        """
        self.last_updater = user
        self.updated_at = datetime.datetime.now()

    @property
    def images(self):
        return list(
            filter(lambda a: a.type == Attachment.TYPE_IMAGE, self.attachment_set.all())
        )

    @property
    def pdfs(self):
        return list(
            filter(lambda a: a.type == Attachment.TYPE_PDF, self.attachment_set.all())
        )

    @property
    def files(self):
        return list(
            filter(lambda a: a.type == Attachment.TYPE_FILE, self.attachment_set.all())
        )

    @property
    def assignee(self):
        if self.stage and self.stage.assignee:
            return self.stage.assignee
        elif self.editor:
            return self.editor
        else:
            return None

    @property
    def prev_stage_assignee(self):
        if self.stage.prev_stage and self.stage.prev_stage.assignee:
            return self.stage.prev_stage.assignee
        else:
            return self.editor

    @property
    def next_stage_assignee(self):
        if self.stage.next_stage and self.stage.next_stage.assignee:
            return self.stage.next_stage.assignee
        else:
            return self.editor

    @property
    def comments_(self):
        return self.comments.order_by('created_at').all

    @property
    def has_text(self):
        return self.xmd is not None and self.xmd != ''

    class Meta:
        permissions = (
            ('recieve_post_email_updates', 'Recieve email updates for Post'),
            ('edit_extended_post_attrs', 'Edit extended Post attributes'),
        )

    @property
    def is_overdue(self):
        if self.stage.slug in ('vault', 'published'):
            return False
        return timezone.now() > self.finished_at

    @property
    def description_html(self):
        return render_md(self.description, render_lead=False)

    @property
    def wp_id(self):
        return self.meta.get('wpid')

    def build_xmd_blob(self, prepared_xmd: str) -> str:
        """Attaches CSS to XMD for remote CMS

            >>> self.css ='foo'
            'foo'

            >>> xmd = 'bar'
            'bar'

            >>> self.build_xmd_blob(self.xmd)
            %style type=text/css
            foo
            %style
            bar
        """
        OPENING_STYLE_TAG = '%style type=text/css'
        CLOSING_STYLE_TAG = '%style'

        if not prepared_xmd:
            return prepared_xmd

        if not self.css:
            return prepared_xmd

        return '{}\n{}\n{}\n{}'.format(
            OPENING_STYLE_TAG,
            self.css,
            CLOSING_STYLE_TAG,
            prepared_xmd
        )

    def upload(self):
        """Uploads post, metadata and attachments to remote CMS

        * self images uploaded to S3.
        * self content uploaded to WP with uploaded S3 images urls

        This method is long-running, can cause time-outs
        and should be run ONLY in async tasks with post lock.

            with Lock():
                post.upload()
        """
        if not self.wp_id:
            return

        for image in self.images:
            image.upload_to_storage()

        # Upload to external DB
        prepared_xmd = replace_images_paths(
            xmd=self.xmd,
            attachments=self.images,
            mapper=s3_image_mapper,
        )

        update_ext_db_xmd(
            self.wp_id,
            xmd=prepared_xmd, title=str(self), css=self.css,
        )


class Attachment(AbstractBase):
    TYPE_IMAGE = 0
    TYPE_PDF = 1
    TYPE_FILE = 2
    TYPE_CHOICES = ((TYPE_IMAGE, 'Image'), (TYPE_PDF, 'PDF'), (TYPE_FILE, 'File'))
    type = models.SmallIntegerField(choices=TYPE_CHOICES, default=TYPE_IMAGE)

    original_filename = models.CharField(null=False, blank=False, max_length=255)
    file = models.FileField(upload_to='attachments/%Y/%m/%d/', max_length=2048)
    user = models.ForeignKey(User, on_delete=models.CASCADE)
    post = models.ForeignKey(Post, on_delete=models.CASCADE)
    meta = JSONField(default=dict)

    S3_BUCKET_URL = 'https://{}.s3.eu-central-1.amazonaws.com'.format(
        os.environ.get('S3_BUCKET_NAME', '')
    )

    @property
    def s3_full_url(self):
        return '{}/{}'.format(self.S3_BUCKET_URL, self._build_s3_path())

    def _build_s3_path(self) -> str:
        """Returns S3 object key path for upload.
        """
        dirname = os.path.dirname(self.file.name)
        basename = os.path.basename(self.file.name)
        hashed_dirname = hashlib.md5(dirname.encode()).hexdigest()

        return '{}/{}/{}'.format(
            hashed_dirname,
            self.id,
            basename
        )

    def _guess_self_mimetype(self) -> tp.Optional[str]:
        try:
            return mimetypes.guess_type(self.file.path, strict=False)[0]
        except ValueError as exc:
            return None

    def _upload_to_s3(self):
        with S3Client() as (s3, S3_BUCKET_NAME):
            object_name = self._build_s3_path()
            try:
                s3_object = s3.Object(S3_BUCKET_NAME, object_name)
                s3_object.upload_file(
                    self.file.path,
                    ExtraArgs={
                        'ACL': 'public-read',
                        'ContentType': self._guess_self_mimetype()
                    })
            except ClientError as e:
                logging.error(e)
                return False
            return True

    def upload_to_storage(self, storage_type: StorageType = StorageType.S3) -> None:
        if storage_type == StorageType.S3:
            self._upload_to_s3()


class Comment(AbstractBase):
    SYSTEM_ACTION_SET_STAGE = 5
    SYSTEM_ACTION_UPDATE = 10
    SYSTEM_ACTION_CHANGE_META = 15
    SYSTEM_ACTION_CHOICES = (
        (SYSTEM_ACTION_SET_STAGE, 'Set stage'),
        (SYSTEM_ACTION_UPDATE, 'Update'),
        (SYSTEM_ACTION_CHANGE_META, 'Change meta'),
    )

    TYPE_SYSTEM = 5
    TYPE_PRIVATE = 10
    TYPE_PUBLIC = 15
    TYPE_CHOICES = (
        (TYPE_SYSTEM, 'system'),
        (TYPE_PRIVATE, 'private'),
        (TYPE_PUBLIC, 'public'),
    )
    text = models.TextField(blank=True)
    type = models.SmallIntegerField(choices=TYPE_CHOICES, default=TYPE_PRIVATE)
    user = models.ForeignKey(User, on_delete=models.CASCADE)

    content_type = models.ForeignKey(ContentType, on_delete=models.CASCADE)
    object_id = models.PositiveIntegerField()
    commentable = GenericForeignKey('content_type', 'object_id')
    meta = JSONField(default=dict)

    def __str__(self):
        return '%s, %s:%s...' % (self.user_id, self.type, self.text[0:50])

    @property
    def html(self):
        return render_md(self.text, render_lead=False)

    @property
    def changelog(self):
        try:
            md = '\n'.join(self.meta['comment']['changelog'])
        except Exception:
            md = ''

        return render_md(md, render_lead=False)


class Vote(AbstractBase):
    SCORE_0 = 0
    SCORE_25 = 25
    SCORE_50 = 50
    SCORE_75 = 75
    SCORE_100 = 100
    SCORE_CHOICES = (
        (SCORE_0, 'Против таких статей в «Хакере»'),
        (SCORE_25, 'Не верю, что выйдет хорошо'),
        (SCORE_50, 'Тема нормальная, но не для меня'),
        (SCORE_75, 'Почитал бы, встретив в журнале'),
        (SCORE_100, 'Ради таких статей мог бы подписаться'),
    )
    score = models.SmallIntegerField(choices=SCORE_CHOICES, default=SCORE_50)
    user = models.ForeignKey(User, on_delete=models.CASCADE)
    idea = models.ForeignKey(Idea, on_delete=models.CASCADE, related_name='votes')

    @property
    def score_humanized(self):
        return self.__class__.SCORE_CHOICES


def users_with_perm(perm_name: str, include_superuser: bool = True) -> List[User]:
    """Get all users by full permission name

    :param perm_name: permission name without app name
    :param include_superuser:
    :return:
    """
    return User.objects.filter(
        Q(is_superuser=include_superuser)
        | Q(user_permissions__codename=perm_name)
        | Q(groups__permissions__codename=perm_name)
    ).distinct()


@receiver(post_save, sender=User)
def create_user_profile(sender, instance, created, **kwargs):
    if created:
        Profile.objects.create(user=instance)


@receiver(post_save, sender=User)
def save_user_profile(sender, instance, **kwargs):
    instance.profile.save()


def _render_with_external_parser(
        id: int, xmd: str, paywall_tag_html: str = Post.PAYWALL_NOTICE_RENDERED
) -> tp.Optional[str]:
    FAILBACK_SYNTAX_LANG = 'cpp'

    if not xmd:
        return None

    if not config.EXTERNAL_PARSER_URL:
        return None

    try:
        request_payload: tp.Dict[str, str] = {
            'id': id,
            'md': xmd,
            'lang': FAILBACK_SYNTAX_LANG,
            'xakepcut': paywall_tag_html,
        }
        request_headers: tp.Dict[str, str] = {  # unusued
            'content-type': 'application/x-www-form-urlencoded; charset=utf-8'
        }
        response = requests.post(
            config.EXTERNAL_PARSER_URL, data=request_payload
        )
        return response.text

    except Exception as exc:
        # TODO: add logger, no need to handle
        return None


@receiver(pre_save, sender=Post)
def render_xmd(sender, instance, **kwargs):
    if not instance.has_text:
        return

    prepared_xmd: str = replace_images_paths(
        instance.xmd, instance.images, mapper=plan_image_mapper
    )
    instance.html = _render_with_external_parser(
        instance.id, prepared_xmd, paywall_tag_html=Post.PAYWALL_NOTICE_RENDERED
    )

    if not instance.html:
        instance.html = render_md(instance.xmd, attachments=instance.images)

    if Post.PAYWALL_NOTICE_HEAD in instance.html:
        instance.is_paywalled = True
    else:
        instance.is_paywalled = False



# Register your models here.
from django.contrib import admin
from django.contrib.auth.admin import UserAdmin as BaseUserAdmin
from django_admin_listfilter_dropdown.filters import RelatedDropdownFilter

from magplan.models import User, Post, Comment, Issue, Stage, Magazine, Section, Idea, Attachment


class UserAdmin(BaseUserAdmin):
    ordering = ['profile__l_name']

    list_display = ['profile__l_name', 'profile__f_name', 'email', ]

    def profile__f_name(self, obj):
        return obj.profile.f_name

    profile__f_name.short_description = 'First name'
    profile__f_name.admin_order_field = 'profile__f_name'

    def profile__l_name(self, obj):
        return obj.profile.l_name

    profile__l_name.admin_order_field = 'profile__l_name'
    profile__l_name.short_description = 'Last name'

    list_filter = (
        'groups__name', 'is_staff', 'is_superuser', 'is_active',
    )


class PostAdmin(admin.ModelAdmin):
    ordering = ['-created_at']

    list_filter = (
        ('stage', RelatedDropdownFilter),
        ('editor', RelatedDropdownFilter),
    )

    list_display = ('title', 'created_at', 'stage', 'section', 'editor')


class AttachmentAdmin(admin.ModelAdmin):
    list_display = ['type', 'file', 'user', 'post']
    list_filter = ['type']


class IdeaAdmin(admin.ModelAdmin):
    pass


class CommentAdmin(admin.ModelAdmin):
    list_filter = ['type']


class StageAdmin(admin.ModelAdmin):
    list_display = ['title', 'duration', 'sort', 'skip_notification', ]


class SectionAdmin(admin.ModelAdmin):
    list_display = ['title', 'slug', 'sort', 'is_archived', 'is_whitelisted', ]
    list_filter = ['is_archived', 'is_whitelisted', ]


admin.site.register(User, UserAdmin)
admin.site.register(Post, PostAdmin)
admin.site.register(Comment, CommentAdmin)
admin.site.register(Issue)
admin.site.register(Stage, StageAdmin)
admin.site.register(Magazine)
admin.site.register(Section, SectionAdmin)
admin.site.register(Idea, IdeaAdmin)
admin.site.register(Attachment, AttachmentAdmin)



import typing as tp

from mistune import Renderer


class XMDRenderer(Renderer):
    def __init__(self, image_mapper: tp.Callable, attachments: tp.List = None, *args, **kwargs):
        self.attachments = attachments or []
        self.image_mapper = image_mapper

        super(XMDRenderer, self).__init__(*args, **kwargs)

    def image(self, src, title, alt_text):
        # Map human-readable filename to urlencoded one
        urlencoded_filename = self.image_mapper(src, self.attachments)

        html = (
            '<figure>'
            '<img conf="%s" alt="%s" />'
            '<figcaption>%s</figcaption>'
            '</figure>'
        )

        return html % (urlencoded_filename, alt_text, alt_text)

    def panel_block_www_start(self):
        html = (
            '<div class="panel www">'
            '<div class="www-inner">'
            '<h3>www</h3>'
        )
        return html

    def panel_block_info_start(self):
        html = (
            '<div class="panel info">'
            '<div class="info-inner">'
            '<h3>info</h3>'
        )
        return html

    def panel_block_warning_start(self):
        html = (
            '<div class="panel warning">'
            '<div class="warning-inner">'
            '<h3>warning</h3>'
        )
        return html

    def panel_block_greeting_start(self):
        html = (
            '<div class="panel greeting">'
            '<div class="greeting-inner">'
            '<h3>greeting</h3>'
        )
        return html

    def panel_block_term_start(self):
        html = (
            '<div class="panel term">'
            '<div class="term-inner">'
            # '<h3>term</h3>'
        )
        return html

    def panel_block_default_start(self, text):
        html = (
            '<div class="panel default">'
            '<div class="default-inner">'
            '<h3>%s</h3>'
        )
        return html % text

    def panel_block_term_code(self, content):
        html = (
            '<pre><code>'
            '%s'
            '</code></pre>'
        )
        return html % content

    def panel_block_end(self):
        html = '</div></div>'
        return html

    def lead_start(self) -> str:
        html = '<div class=lead>'

        return html

    def lead_end(self) -> str:
        html = '</div>'

        return html

    def paywall(self) -> str:
        html = (
            '<div class="paywall-notice">'
            'Продолжение статьи доступно только продписчикам'
            '</div>'
        )

        return html



from django.utils import translation


class SetLanguageMiddleware:
    def __init__(self, get_response):
        self.get_response = get_response
        # One-time configuration and initialization.

    def __call__(self, request):
        # Set user language from settings
        try:
            user_language = request.user.preferences.get('magplan__ui_language')
            translation.activate(user_language)
            request.session[translation.LANGUAGE_SESSION_KEY] = user_language
        except:
            # HACK: should not raise I18n errors if Accept-Language
            # or settings is not configured
            pass

        response = self.get_response(request)

        return response



from typing import List


def get_attachment_original_filename(filename: str, attachments: List) -> str:
    """Get actual urlencoded filename from database
    by human-readable attachment filename

    """
    # Find requested attachment from post attachments
    attachments_ = [
        attachment
        for attachment in attachments
        if attachment.original_filename == filename
    ]
    if not attachments_:
        return ''

    return attachments_[0].file.url



from django import forms
from dynamic_preferences.preferences import Section
from dynamic_preferences.types import ChoicePreference
from dynamic_preferences.users.registries import user_preferences_registry

plan = Section('magplan')


@user_preferences_registry.register
class PlanUILanguage(ChoicePreference):
    name = 'ui_language'
    section = plan
    choices = [
        ('en', 'English'),
        ('ru', 'Русский')
    ]
    default = 'ru'
    widget = forms.Select(attrs={'class': 'form-control'})
    verbose_name = 'Язык интерфейса плана'


@user_preferences_registry.register
class PostCommentNotificationLevel(ChoicePreference):
    name = 'post_comment_notification_level'
    section = plan
    choices = [
        ('none', 'Не присылать уведомлений о комментариях'),
        ('related', 'Присылать только те, что относятся ко мне'),
        ('all', 'Присылать все уведомления'),
    ]
    default = 'related'
    widget = forms.Select(attrs={'class': 'form-control'})
    verbose_name = 'Уведомления о комментариях к посту'


@user_preferences_registry.register
class IdeaCommentNotificationLevel(ChoicePreference):
    name = 'idea_comment_notification_level'
    section = plan
    choices = [
        ('none', 'Не присылать уведомлений о комментариях'),
        ('related', 'Присылать только те, что относятся ко мне'),
        ('all', 'Присылать все уведомления'),
    ]
    default = 'related'
    widget = forms.Select(attrs={'class': 'form-control'})
    verbose_name = 'Уведомления о комментариях к идее'

@user_preferences_registry.register
class XMDEditorType(ChoicePreference):
    name = 'xmd_editor_type'
    section = plan
    choices = [
        ('plain', 'Plain-text редактор'),
        ('markdown', 'Markdown-редактор'),
    ]
    default = 'markdown'
    widget = forms.Select(attrs={'class': 'form-control'})
    verbose_name = 'Редактор для статей в XMD'

@user_preferences_registry.register
class NewIdeaNotification(ChoicePreference):
    name = 'new_idea_notification'
    section = plan
    choices = [
        ('yes', 'Да'),
        ('no', 'Нет'),
    ]
    default = 'yes'
    widget = forms.Select(attrs={'class': 'form-control'})
    verbose_name = 'Присылать уведомелния о новых идеях?'



from django.contrib import messages
from django.contrib.auth.decorators import login_required
from django.contrib.auth.decorators import permission_required
from django.db.models import Q
from django.shortcuts import render, redirect

from magplan.models import User, Post
from magplan.forms import UserModelForm, ProfileModelForm


@login_required
@permission_required('main.manage_authors')
def index(request):
    users = User.objects.prefetch_related('profile').order_by('profile__l_name').all()
    return render(request, 'magplan/authors/index.html', {
        'users': users
    })


@login_required
@permission_required('main.manage_authors')
def new(request):
    if request.method == 'POST':
        user_form = UserModelForm(request.POST)
        profile_form = ProfileModelForm(request.POST)

        if user_form.is_valid() and profile_form.is_valid():
            user = user_form.save(commit=False)
            # Authors should be inactive by default
            user.is_active = False
            user.save()

            profile_form = ProfileModelForm(request.POST, instance=user.profile)
            profile_form.save()

            messages.add_message(request, messages.INFO, f'Автор «{user}» успешно создан')

            return redirect('authors_edit', user.id)

    else:
        user_form = UserModelForm()
        profile_form = ProfileModelForm()

    return render(request, 'magplan/authors/new.html', {
        'user_form': user_form,
        'profile_form': profile_form,
    })


@login_required
@permission_required('main.manage_authors')
def show(request, user_id):
    user = User.objects.get(id=user_id)
    posts = (Post.objects.filter(Q(authors=user) | Q(editor=user))
             .prefetch_related('authors', 'editor', 'stage', 'section')
             .exclude(stage__slug='vault')
             .order_by('created_at'))
    return render(request, 'magplan/authors/show.html', {
        'user': user,
        'posts': posts,
    })


@login_required
@permission_required('main.manage_authors')
def edit(request, user_id):
    user = User.objects.get(id=user_id)

    if request.method == 'POST':
        user_form = UserModelForm(request.POST, instance=user)
        profile_form = ProfileModelForm(request.POST, instance=user.profile)

        if user_form.is_valid() and profile_form.is_valid():
            user_form.save()
            profile_form.save()

            messages.add_message(request, messages.INFO, f'Автор «{user}» успешно отредактирован')

    else:
        user_form = UserModelForm(instance=user)
        profile_form = ProfileModelForm(instance=user.profile)

    return render(request, 'magplan/authors/edit.html', {
        'user_form': user_form,
        'profile_form': profile_form,
        'user': user,
    })



# Generated by Django 3.1.2 on 2020-11-15 11:40

import django.contrib.auth.models
import django.contrib.postgres.fields.jsonb
from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('magplan', '0001_initial'),
        ('contenttypes', '0002_remove_content_type_name'),
    ]

    operations = [
        migrations.CreateModel(
            name='User',
            fields=[
                ('user_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to=settings.AUTH_USER_MODEL)),
                ('meta', django.contrib.postgres.fields.jsonb.JSONField(default=dict)),
            ],
            options={
                'permissions': (('access_magplan', 'Can access magplan'), ('manage_authors', 'Can manage authors')),
            },
            bases=('main.user',),
            managers=[
                ('objects', django.contrib.auth.models.UserManager()),
            ],
        ),
        migrations.CreateModel(
            name='Widget',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('_old_id', models.PositiveIntegerField(blank=True, null=True)),
                ('content', models.TextField()),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Widgetype',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('_old_id', models.PositiveIntegerField(blank=True, null=True)),
                ('slug', models.SlugField(max_length=255)),
                ('title', models.CharField(max_length=255)),
                ('meta', django.contrib.postgres.fields.jsonb.JSONField(default=dict)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Vote',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('_old_id', models.PositiveIntegerField(blank=True, null=True)),
                ('score', models.SmallIntegerField(choices=[(0, 'Против таких статей в «Хакере»'), (25, 'Не верю, что выйдет хорошо'), (50, 'Тема нормальная, но не для меня'), (75, 'Почитал бы, встретив в журнале'), (100, 'Ради таких статей мог бы подписаться')], default=50)),
                ('idea', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='votes', to='magplan.idea')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='magplan.user')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='stage',
            name='assignee',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='magplan.user'),
        ),
        migrations.AddField(
            model_name='stage',
            name='next_stage',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='p_stage', to='magplan.stage'),
        ),
        migrations.AddField(
            model_name='stage',
            name='prev_stage',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='n_stage', to='magplan.stage'),
        ),
        migrations.AddField(
            model_name='profile',
            name='user',
            field=models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, related_name='profile', to='magplan.user'),
        ),
        migrations.AddField(
            model_name='post',
            name='authors',
            field=models.ManyToManyField(to='magplan.User', verbose_name='Авторы'),
        ),
        migrations.AddField(
            model_name='post',
            name='editor',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='edited', to='magplan.user', verbose_name='Редактор'),
        ),
        migrations.AddField(
            model_name='post',
            name='issues',
            field=models.ManyToManyField(related_name='posts', to='magplan.Issue', verbose_name='Выпуски'),
        ),
        migrations.AddField(
            model_name='post',
            name='last_updater',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='posts_updated', to='magplan.user', verbose_name='Кто последний обновлял'),
        ),
        migrations.AddField(
            model_name='post',
            name='postype',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='magplan.postype'),
        ),
        migrations.AddField(
            model_name='post',
            name='section',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='magplan.section', verbose_name='Раздел'),
        ),
        migrations.AddField(
            model_name='post',
            name='stage',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='magplan.stage', verbose_name='Этап'),
        ),
        migrations.AddField(
            model_name='issue',
            name='magazine',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='magplan.magazine'),
        ),
        migrations.AddField(
            model_name='idea',
            name='authors',
            field=models.ManyToManyField(blank=True, related_name='authors', to='magplan.User', verbose_name='Авторы'),
        ),
        migrations.AddField(
            model_name='idea',
            name='editor',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='editor', to='magplan.user'),
        ),
        migrations.AddField(
            model_name='idea',
            name='post',
            field=models.OneToOneField(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='magplan.post'),
        ),
        migrations.AddField(
            model_name='comment',
            name='content_type',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='contenttypes.contenttype'),
        ),
        migrations.AddField(
            model_name='comment',
            name='user',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='magplan.user'),
        ),
        migrations.AddField(
            model_name='attachment',
            name='post',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='magplan.post'),
        ),
        migrations.AddField(
            model_name='attachment',
            name='user',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='magplan.user'),
        ),
    ]



# Create your tasks here
from __future__ import absolute_import, unicode_literals

import logging
import os
from typing import Set

import html2text
from celery import shared_task
from django.contrib.auth.models import User
from django.contrib.contenttypes.models import ContentType
from django.core.mail import EmailMultiAlternatives
from django.template.loader import render_to_string

from magplan.conf import settings as config
from magplan.models import Comment
from magplan.tasks.utils import _can_recieve_notification, _get_whitelisted_recipients

RECIEVE_NOTIFICATIONS_PERMISSION = 'main.recieve_post_email_updates'
NOTIFICATION_LEVEL_PREFERENCE = 'post_comment_notification_level'


def _get_involved_users(comment: Comment) -> set:
    """Return all users, who involved in working
    on current post

    :param comment: Comment for post, for which we count involved users
    :return: Users set
    """
    post = comment.commentable

    # Django objects are hashable, so we can add
    # any probabl reciever to common set, and then
    # filter by permissions
    users = set()

    # Add current stage assignee
    users.add(post.assignee)

    # Add post authors and editors
    users.add(post.editor)
    users.update([author for author in post.authors.all()])

    # Add all previous commenters in the tread
    previous_comments = Comment.objects.filter(
        content_type=ContentType.objects.get_for_model(post), object_id=post.id, type=Comment.TYPE_PRIVATE
    ).exclude(id=comment.id)

    users.update([pc.user for pc in previous_comments])

    return users


def _get_recipients(comment: Comment) -> Set[User]:
    """Build final list of notification recipients
    for comments.

    Includes:

    — those, who should receive all notifications;
    — those, who comment is related to.

    Excludes users, who restricted any post comments
    notifications.

    :param comment: Comment, for which we build
                    recipient list for
    :return: Final set of actual recipients
    """
    recipients = set()

    involved_users = _get_involved_users(comment)
    recipients.update(involved_users)

    whitelisted_recipients = _get_whitelisted_recipients(NOTIFICATION_LEVEL_PREFERENCE)
    recipients.update(whitelisted_recipients)

    # Remove users, who cannot receive notifications
    # due to their permissions, comment ownership or settings
    recipients = {
        recipient
        for recipient in recipients
        if _can_recieve_notification(
            recipient, comment, RECIEVE_NOTIFICATIONS_PERMISSION, NOTIFICATION_LEVEL_PREFERENCE
        )
    }

    return recipients


def _send_email(comment, recipients):
    subject = f"Комментарий к посту «{comment.commentable}» от {comment.user}"
    commentable_type = "post" if comment.commentable.__class__.__name__ == "Post" else "idea"
    html_content = render_to_string(
        "email/new_comment.html",
        {"comment": comment, "commentable_type": commentable_type, "APP_URL": os.environ.get("APP_URL", None)},
    )
    text_content = html2text.html2text(html_content)
    msg = EmailMultiAlternatives(subject, text_content, config.PLAN_EMAIL_FROM, recipients)
    msg.attach_alternative(html_content, "text/html")
    msg.send()


@shared_task
def send_post_comment_notification(comment_id: int) -> None:
    """Send email notification for post comment

    Notification is sent to:

    - comment post current stage assignee
    - comment post authors and editor
    - everyone, who previously commented comment post

    But only, if users have recieve_post_email_updates
    permission and not comment author.
    """
    logger = logging.getLogger()
    logger.info('Sending notifications for comment #%s', comment_id)

    # Task will fail if comment is not found
    comment = Comment.objects.get(id=comment_id)

    recipients = _get_recipients(comment)

    if not recipients:
        return

    # Transform to plain email list
    recipients_emails = {recipient.email for recipient in recipients}

    _send_email(comment, recipients_emails)



import logging

from dynamic_preferences.users.models import UserPreferenceModel

from magplan.conf import settings as config
from magplan.models import Comment, User


def _get_whitelisted_recipients(pref_name: str) -> set:
    """Get all users, who should receive all
    post comments notifications

    NB: relies on default setting == 'related'!

    :return: Set of users, who we should send
             every comment
    """

    preferences = UserPreferenceModel.objects.prefetch_related('instance').filter(
        section='magplan', name=pref_name, raw_value='all'
    )

    return {p.instance for p in preferences}


def _can_recieve_notification(
        user: User, comment: Comment, perm_name: str, pref_name: str
) -> bool:
    logger = logging.getLogger()
    logger.debug('Checking permissions for recipient %s...' % user.email)

    if not user.has_perm(perm_name):
        logger.debug(
            'Recipient %s decliened as it has no permission to receive post updates.'
            % user.email
        )
        return False

    if user == comment.user:
        logger.debug('Recipient %s decliened as it\'s comment author' % user.email)
        return False

    if config.SYSTEM_USER_ID and user.id == config.SYSTEM_USER_ID:
        logger.debug('Recipient %s decliened as it\'s system user' % user.email)
        return False

    # Disable notification for restricted notification users.
    # This util function is used only in magplan app
    # so it's better to hardcode section name
    # for better re-usabillty
    if user.preferences['magplan__' + pref_name] == 'none':
        logger.debug(
            'Recipient %s decliened as it has turned notifications off' % user.email
        )
        return False

    return True



from django.conf import settings as django_settings

SYSTEM_USER_ID = getattr(django_settings, 'SYSTEM_USER_ID', 'foo@bar.baz')
PLAN_EMAIL_FROM = getattr(django_settings, 'PLAN_EMAIL_FROM', 'foo@bar.baz')
EXTERNAL_PARSER_URL = getattr(django_settings, 'EXTERNAL_PARSER_URL', None)
PLAN_POSTS_INSTANCE_CHUNK = getattr(django_settings, 'PLAN_POSTS_INSTANCE_CHUNK', None)
PLAN_EMAIL_SUBJECT_PREFIX = getattr(django_settings, 'PLAN_EMAIL_SUBJECT_PREFIX', '[magplan]')

APP_HOST = getattr(django_settings, 'APP_HOST', 'magpplan.example.com')
APP_URL = getattr(django_settings, 'APP_URL', 'https://magpplan.example.com')
APP_ENV = getattr(django_settings, 'APP_ENV', 'PRODUCTION')

SSH_HOST = getattr(django_settings, 'SSH_HOST', '127.0.0.1')
SSH_PORT = getattr(django_settings, 'SSH_PORT', 22)
SSH_USER = getattr(django_settings, 'SSH_USER', 'user')
SSH_PASS = getattr(django_settings, 'SSH_PASS', 'password')

EXT_DB_HOST = getattr(django_settings, 'EXT_DB_HOST', '127.0.0.1')
EXT_DB_PORT = getattr(django_settings, 'EXT_DB_PORT', '5432')
EXT_DB_NAME = getattr(django_settings, 'EXT_DB_NAME', 'database')
EXT_DB_USER = getattr(django_settings, 'EXT_DB_USER', 'db_user')
EXT_DB_PASS = getattr(django_settings, 'EXT_DB_PASS', 'db_pass')

S3_ACCESS_KEY = getattr(django_settings, 'S3_ACCESS_KEY', 's3_access_key')
S3_SECRET_KEY = getattr(django_settings, 'S3_SECRET_KEY', 's3_secret_key')
S3_BUCKET_NAME = getattr(django_settings, 'S3_BUCKET_NAME', 's3_bucket_name')



from django.shortcuts import render
from magplan.models import Issue, Magazine
from django.contrib.auth.decorators import login_required
from magplan.forms import IssueModelForm
from django.shortcuts import render, HttpResponse, redirect


# Create your views here.
@login_required
def index(request):
    issues = Issue.objects.all().order_by('-number').prefetch_related('magazine')

    return render(request, 'magplan/issues/index.html', {
        'issues': issues,
    })


@login_required
def create(request):
    form = IssueModelForm()
    if request.method == 'POST':
        form = IssueModelForm(request.POST)
        if form.is_valid():
            issue = form.save(commit=False)
            issue.magazine = Magazine.objects.first()
            issue.save()
            return redirect('issues_index')

    return render(request, 'magplan/issues/new.html', {
        'form': form,
    })


@login_required
def show(request, issue_id):
    issue = Issue.objects.get(id=issue_id)
    posts = issue.posts.all().order_by('section__sort').prefetch_related('authors', 'editor', 'stage',
                                                                         'section').exclude(stage__slug='vault')

    return render(request, 'magplan/issues/show.html', {
        'issue': issue,
        'posts': posts,
        'stats': {
            'has_text': len(list(filter(lambda p: (p.xmd is not None) and (p.xmd != ''), posts))),
            'incomplete': len(list(filter(lambda p: p.stage.slug != 'published', posts))),
        }
    })



from django.core.management.base import BaseCommand
from magplan.models import *
from tqdm import tqdm


class Command(BaseCommand):
    help = 'Re-renders all posts content'

    def handle(self, *args, **options):
        posts = Post.objects.filter(xmd__isnull=False).exclude(xmd__exact='')
        for post in tqdm(posts):
            post.save()


from django.shortcuts import render
from magplan.models import Stage
from django.contrib.auth.decorators import login_required


# Create your views here.
@login_required
def index(request):
    return render(request, 'magplan/stages/index.html', {
        'stages': (Stage.objects.order_by('sort').all()
                   .prefetch_related('prev_stage', 'next_stage', 'assignee'))
    })



import re
import typing as tp
from contextlib import contextmanager

import pymysql
from django.conf import settings
from sshtunnel import SSHTunnelForwarder

from magplan.xmd.mappers import s3_public_mapper

img_pattern = re.compile(
    '^'  # Only from beginning
    ' *'  # Some leading spaces may persist
    '!\[.*?\]\((.+)\)'  # MD image pattern
    ' *?'  # Some ending spaces may persis
    '$'  # Capture urls only from last section
)

url_replace_pattern = re.compile(
    '\]\((.+)\)$'  # FIXME: match real last () group
)


@contextmanager
def Lock(post):
    post.is_locked = True
    post.save()

    yield

    post.is_locked = False
    post.save()


def replace_images_paths(xmd: str, attachments: tp.List, mapper: tp.Callable = None) -> str:
    """Replaces all links to local images
    to absolute paths with mapper function.

    Can parse both regualar images and galleries.
    If provided link not found, it's left intact.

    Examples:

        ![](image1.jpg)
        ![](image1.jpg,image2.jpg)
    """
    if not mapper:
        mapper = s3_public_mapper

    result_lines = []

    for line in xmd.splitlines():
        # Extract image
        url_chunk = re.findall(img_pattern, line)
        if not url_chunk:  # If no image in line
            result_lines.append(line)
            continue

        # Parse all images for galleries one
        filenames = [
            filename.strip()
            for filename in url_chunk[0].split(',')  # First and only match group
        ]
        absolute_url_chunks = [
            mapper(filename, attachments)
            for filename in filenames
        ]

        # Build result line with new images
        absolute_urs = ']({})'.format(','.join(absolute_url_chunks))
        res_line = re.sub(url_replace_pattern, absolute_urs, line)

        result_lines.append(res_line)

    return '\n'.join(result_lines)


def update_post_meta_field(conn: pymysql.Connection, object_id: int, meta_key: str, meta_value: str):
    with conn.cursor() as cursor:
        retrieve_query = 'SELECT meta_value FROM wp_postmeta WHERE post_id=%s and meta_key=%s limit 1;'
        cursor.execute(retrieve_query, (object_id, meta_key))

        # Update existing if found
        existing_rows = cursor.fetchone()
        if existing_rows:
            update_query = 'UPDATE wp_postmeta SET meta_value=%s WHERE post_id=%s and meta_key=%s;'
        else:
            update_query = 'INSERT INTO wp_postmeta (meta_value, post_id, meta_key) VALUES (%s, %s, %s);'
        cursor.execute(update_query, (meta_value, object_id, meta_key,))

    conn.commit()


def update_post_field(conn: pymysql.Connection, object_id: int, key: str, value: str):
    with conn.cursor() as cursor:
        update_query = f'UPDATE wp_posts SET {key}=%s WHERE ID=%s;'  # Injection-safe as key is trusted
        cursor.execute(update_query, (value, object_id,))

    conn.commit()


def update_ext_db_xmd(post_id: int, *, xmd: str, title: str, css: str) -> None:
    """
    Updates provided kwargs for post with post_id in external DB
    """
    if not all(settings.EXT_DB.get(key) for key in settings.EXT_DB.keys()):
        return

    conf = settings.EXT_DB

    # Prepare settings for SSH tunnel connection
    ssh_conn_args = ((conf['SSH_HOST'], int(conf['SSH_PORT'])),)
    ssh_conn_kwargs = {
        'ssh_username': conf['SSH_USER'],
        'ssh_password': conf['SSH_PASS'],
        'remote_bind_address': (conf['EXT_DB_HOST'], int(conf['EXT_DB_PORT']))
    }

    with SSHTunnelForwarder(*ssh_conn_args, **ssh_conn_kwargs) as tunnel:
        mysql_conn_settings = {
            'host': conf['EXT_DB_HOST'],
            'user': conf['EXT_DB_USER'],
            'passwd': conf['EXT_DB_PASS'],
            'db': conf['EXT_DB_NAME'],
            'port': tunnel.local_bind_port
        }
        conn = pymysql.connect(**mysql_conn_settings)

        if xmd:
            update_post_meta_field(conn, post_id, 'md', xmd)
        if css:
            update_post_meta_field(conn, post_id, 'css', css)
        if title:
            update_post_field(conn, post_id, 'post_title', title)

    conn.close()



from magplan.models import Issue
import os
# from django.contrib.sites.models import Site


def inject_last_issues(request):
    issues = Issue.objects.order_by('-number').all()[:5]
    return {
        'navbar_issues': issues,
    }


def inject_app_url(request):
    # def current_site_failback():
    #     current_site = Site.objects.get_current()
    #     return current_site.domain

    return {
        'APP_URL': os.environ.get('APP_URL', None),
    }


from mistune import Markdown


class ExtendedMarkdown(Markdown):
    def output_panel_block_www(self):
        return self.renderer.panel_block_www_start()

    def output_panel_block_info(self):
        return self.renderer.panel_block_info_start()

    def output_panel_block_warning(self):
        return self.renderer.panel_block_warning_start()

    def output_panel_block_greeting(self):
        return self.renderer.panel_block_greeting_start()

    def output_panel_block_term(self):
        return self.renderer.panel_block_term_start()

    def output_panel_block_term_code(self):
        text = self.token.get('content', '')
        return self.renderer.panel_block_term_code(text)

    def output_panel_block_default(self):
        text = self.token.get('content', '')
        return self.renderer.panel_block_default_start(text)

    def output_panel_block_end(self):
        return self.renderer.panel_block_end()

    def output_lead(self):
        """NOTE: this function should be called output_lead_start,
        but mistune has line:
        
            # sepcial cases
            if t.endswith('_start'):
                t = t[:-6]
        
        which trims token name for some weird reason.
        """
        return self.renderer.lead_start()

    def output_lead_end(self):
        return self.renderer.lead_end()
        
    def output_paywall(self):
        return self.renderer.paywall()



from celery import shared_task

from magplan.models import Idea


@shared_task
def send_idea_notification(idea_id: int) -> None:
    """Send email notification for new idea with vote options

    Notification is sent to:

    - very user with is_active=True

    But only, if users have appropriate permission
    to receive ideas notifications.
    """
    idea: Idea = Idea.objects.get(id=idea_id)
    if not idea:
        return

    idea.send_vote_notifications()



# Create your tasks here
from __future__ import absolute_import, unicode_literals

from celery import shared_task

from magplan.models import Post
from magplan.integrations.posts import Lock


@shared_task
def upload_post_to_wp(post_id: int) -> None:
    post = Post.objects.filter(id=post_id).first()
    if post is None:
        return

    with Lock(post):
        post.upload()



import typing as tp

from magplan.xmd.lexers import PanelBlockLexer
from magplan.xmd.mappers import plan_internal_mapper as default_image_mapper
from magplan.xmd.markdown import ExtendedMarkdown
from magplan.xmd.renderer import XMDRenderer


def render_md(
        md_text: str,
        image_mapper: tp.Callable = default_image_mapper,
        attachments: tp.List = None,
        render_lead=True,
        *args, **kwargs
) -> str:
    """Render markdown chunk with optional assets preprocessing.

    :param md_text: Markdown string to redner
    :param attachments: Attachments instances, used in provided MD
    :return: Rendered HTML string
    """
    renderer = XMDRenderer(image_mapper=image_mapper, attachments=attachments)
    block = PanelBlockLexer()

    # If we don't want to render lead, fake lead existance
    if not render_lead:
        block.has_lead = True

    markdown = ExtendedMarkdown(renderer=renderer, block=block)

    return markdown(md_text)



# Media elements
image_html = f'''
<figure><img src="%s" alt="%s"><figcaption>%s</figcaption></figure>
'''

# Panels
panel_info = f'''<details><summary>%s</summary><p>%s</p></details>'''
panel_no_title = f'''<details><p>%s</p></details>'''
panel_default = f'''<details><summary>%s</summary><p>%s</p></details>'''



# Generated by Django 3.1.2 on 2020-11-15 11:40

import datetime
import django.contrib.postgres.fields.jsonb
from django.db import migrations, models
import django.utils.timezone


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Attachment',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('_old_id', models.PositiveIntegerField(blank=True, null=True)),
                ('type', models.SmallIntegerField(choices=[(0, 'Image'), (1, 'PDF'), (2, 'File')], default=0)),
                ('original_filename', models.CharField(max_length=255)),
                ('file', models.FileField(max_length=2048, upload_to='attachments/%Y/%m/%d/')),
                ('meta', django.contrib.postgres.fields.jsonb.JSONField(default=dict)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Comment',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('_old_id', models.PositiveIntegerField(blank=True, null=True)),
                ('text', models.TextField(blank=True)),
                ('type', models.SmallIntegerField(choices=[(5, 'system'), (10, 'private'), (15, 'public')], default=10)),
                ('object_id', models.PositiveIntegerField()),
                ('meta', django.contrib.postgres.fields.jsonb.JSONField(default=dict)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Idea',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('_old_id', models.PositiveIntegerField(blank=True, null=True)),
                ('title', models.CharField(max_length=255, verbose_name='Заголовок')),
                ('description', models.TextField(verbose_name='Описание')),
                ('approved', models.BooleanField(null=True)),
                ('author_type', models.CharField(choices=[('NO', 'Нет автора'), ('NW', 'Новый автор'), ('EX', 'Существующий автор(ы)')], default='NO', max_length=2, verbose_name='Автор')),
                ('authors_new', models.CharField(blank=True, max_length=255, null=True, verbose_name='Новые автор')),
            ],
            options={
                'permissions': (('edit_extended_idea_attrs', 'Edit extended Idea attributes'), ('recieve_idea_email_updates', 'Recieve email updates for Idea')),
            },
        ),
        migrations.CreateModel(
            name='Issue',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('_old_id', models.PositiveIntegerField(blank=True, null=True)),
                ('number', models.SmallIntegerField(default=0)),
                ('title', models.CharField(max_length=255, null=True)),
                ('description', models.TextField(null=True)),
                ('published_at', models.DateField(default=datetime.date.today)),
            ],
            options={
                'ordering': ['-number'],
            },
        ),
        migrations.CreateModel(
            name='Magazine',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('_old_id', models.PositiveIntegerField(blank=True, null=True)),
                ('slug', models.SlugField(max_length=255)),
                ('title', models.CharField(max_length=255)),
                ('description', models.TextField(blank=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Post',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('_old_id', models.PositiveIntegerField(blank=True, null=True)),
                ('format', models.SmallIntegerField(choices=[(0, 'Default'), (1, 'Featured')], default=0)),
                ('finished_at', models.DateTimeField(default=django.utils.timezone.now, verbose_name='Дедлайн')),
                ('published_at', models.DateTimeField(blank=True, null=True, verbose_name='Дата публикации')),
                ('kicker', models.CharField(blank=True, max_length=255, null=True)),
                ('slug', models.SlugField(blank=True, max_length=255, null=True)),
                ('title', models.CharField(blank=True, max_length=255, null=True, verbose_name='Заголовок статьи')),
                ('description', models.TextField(blank=True, verbose_name='Описание статьи')),
                ('views', models.IntegerField(default=0)),
                ('is_paywalled', models.BooleanField(default=False)),
                ('xmd', models.TextField(null=True)),
                ('html', models.TextField(null=True)),
                ('meta', django.contrib.postgres.fields.jsonb.JSONField(default=dict)),
                ('is_locked', models.BooleanField(default=False)),
                ('css', models.TextField(blank=True, help_text='CSS, который будет применяться к превью статьи', null=True, verbose_name='CSS-стили для статьи')),
            ],
            options={
                'permissions': (('recieve_post_email_updates', 'Recieve email updates for Post'), ('edit_extended_post_attrs', 'Edit extended Post attributes')),
            },
        ),
        migrations.CreateModel(
            name='Postype',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('_old_id', models.PositiveIntegerField(blank=True, null=True)),
                ('slug', models.SlugField(max_length=255)),
                ('title', models.CharField(max_length=255)),
                ('meta', django.contrib.postgres.fields.jsonb.JSONField(default=dict)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Profile',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('_old_id', models.PositiveIntegerField(blank=True, null=True)),
                ('is_public', models.BooleanField(default=False)),
                ('f_name', models.CharField(blank=True, max_length=255, null=True, verbose_name='Имя')),
                ('m_name', models.CharField(blank=True, max_length=255, null=True, verbose_name='Отчество')),
                ('l_name', models.CharField(blank=True, max_length=255, null=True, verbose_name='Фамилия')),
                ('n_name', models.CharField(blank=True, max_length=255, null=True, verbose_name='Ник')),
                ('country', models.SmallIntegerField(choices=[(0, 'Россия'), (1, 'Украина'), (2, 'Беларусь'), (3, 'Казахстан')], default=0, verbose_name='Страна')),
                ('city', models.CharField(blank=True, max_length=255, null=True, verbose_name='Город или поселок')),
                ('notes', models.TextField(blank=True, null=True, verbose_name='Примечания')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Section',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('_old_id', models.PositiveIntegerField(blank=True, null=True)),
                ('slug', models.SlugField(max_length=255)),
                ('title', models.CharField(max_length=255)),
                ('description', models.TextField(null=True)),
                ('sort', models.SmallIntegerField(default=0)),
                ('color', models.CharField(default='000000', max_length=6)),
                ('is_archived', models.BooleanField(default=False)),
                ('is_whitelisted', models.BooleanField(default=False)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Stage',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('_old_id', models.PositiveIntegerField(blank=True, null=True)),
                ('slug', models.SlugField(max_length=255)),
                ('title', models.CharField(max_length=255)),
                ('sort', models.SmallIntegerField(default=0)),
                ('duration', models.SmallIntegerField(blank=True, default=1, null=True)),
                ('skip_notification', models.BooleanField(default=False)),
                ('meta', django.contrib.postgres.fields.jsonb.JSONField(default=dict)),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



from collections import namedtuple

from django import forms
from django.forms import ModelForm
from django_ace import AceWidget

from magplan.models import Idea, Post, Comment, Section, Issue, User, Profile

SelectChoice = namedtuple('SelectChoice', ['slug', 'title'])
IDEA_AUTHOR_SELF_CHOICE = SelectChoice('SELF', 'Напишу сам')


class UserModelForm(ModelForm):
    class Meta:
        model = User
        fields = ('email',)
        widgets = {
            'email': forms.TextInput(attrs={'class': 'form-control', }),
        }


class ProfileModelForm(ModelForm):
    class Meta:
        model = Profile
        fields = ('l_name', 'f_name', 'n_name', 'notes',)
        widgets = {
            'l_name': forms.TextInput(attrs={'class': 'form-control', }),
            'f_name': forms.TextInput(attrs={'class': 'form-control', }),
            'n_name': forms.TextInput(attrs={'class': 'form-control', }),
            'notes': forms.Textarea(attrs={'class': 'form-control', }),
        }

    def __init__(self, *args, **kwargs):
        super(ProfileModelForm, self).__init__(*args, **kwargs)
        for f in ('l_name', 'f_name', 'n_name', 'notes',):
            self.fields[f].required = False


class IdeaAuthorTypeChoiceField(forms.ChoiceField):
    def validate(self, value):
        """Custom field, which allows virtual SELF field value"""
        if value == IDEA_AUTHOR_SELF_CHOICE.slug:
            return

        super().validate(value)


class IdeaModelForm(ModelForm):
    # Add virtual field to extend default widget choices

    EXTENDED_AUTHOR_TYPE_CHOICES = (
        IDEA_AUTHOR_SELF_CHOICE, *reversed(Idea.AUTHOR_TYPE_CHOICES)
    )

    author_type = IdeaAuthorTypeChoiceField(
        choices=EXTENDED_AUTHOR_TYPE_CHOICES,
        widget=forms.Select(
            attrs={'class': 'form-control'},
        ),
        label=Idea._meta.get_field('author_type').verbose_name
    )

    class Meta:
        model = Idea
        fields = ['title', 'description', 'author_type', 'authors_new', 'authors']
        widgets = {
            'title': forms.TextInput(attrs={'class': 'form-control', }),
            'description': forms.Textarea(attrs={'class': 'form-control', 'rows': 5}),
            'authors_new': forms.TextInput(attrs={'class': 'form-control'}),
            'authors': forms.SelectMultiple(attrs={
                'class': 'form-control live_multiselect',
                'data-url': 'api/users/search',
            }),
        }

    def clean_author_type(self):
        data = self.cleaned_data['author_type']
        if data == IDEA_AUTHOR_SELF_CHOICE.slug:
            return Idea.AUTHOR_TYPE_EXISTING

        return data


class IssueModelForm(ModelForm):
    class Meta:
        model = Issue
        fields = ('number', 'title', 'description', 'published_at',)
        widgets = {
            'number': forms.TextInput(attrs={'class': 'form-control', }),
            'title': forms.TextInput(attrs={'class': 'form-control', }),
            'description': forms.Textarea(attrs={'class': 'form-control', 'rows': 5}),
            'published_at': forms.DateInput(attrs={'class': 'form-control date_picker', }),
        }


class PostMetaForm(ModelForm):
    wp_id = forms.IntegerField(widget=forms.TextInput(attrs={'class': 'form-control', }))
    editor = forms.ModelChoiceField(queryset=(
        User.objects.filter(is_active=True).prefetch_related('profile').order_by('profile__l_name').all()
    ),
        label="Редактор",
        empty_label=None,
        widget=forms.Select(attrs={'class': 'form-control', 'rows': 5})
    )

    class Meta:
        model = Post
        fields = ('issues', 'editor', 'finished_at', 'published_at', 'css')
        # css = forms.CharField(widget=AceWidget)

        widgets = {
            'issues': forms.SelectMultiple(attrs={
                'class': 'form-control live_multiselect wiih_suggestions',
                'data-url': 'api/issues/search',
            }),
            'editor': forms.Select(attrs={'class': 'form-control', }),
            'finished_at': forms.DateInput(attrs={'class': 'form-control date_picker', }),
            'published_at': forms.DateInput(attrs={'class': 'form-control date_picker', }),
            'css': AceWidget(
                mode='css', theme='textmate', showinvisibles=True, toolbar=False
            ),
        }

    def __init__(self, *args, **kwargs):
        super(PostMetaForm, self).__init__(*args, **kwargs)
        self.fields['wp_id'].required = False


class PostBaseModelForm(ModelForm):
    section = forms.ModelChoiceField(
        queryset=Section.objects.filter(is_whitelisted=False, is_archived=False),
        label="Рубрика",
        empty_label=None,
        widget=forms.Select(attrs={'class': 'form-control', 'rows': 5})
    )

    class Meta:
        model = Post
        fields = ('title', 'description', 'issues', 'authors', 'finished_at', 'section',)
        widgets = {
            'title': forms.TextInput(attrs={'class': 'form-control', }),
            'description': forms.Textarea(attrs={'class': 'form-control', }),
            'issues': forms.SelectMultiple(attrs={
                'class': 'form-control live_multiselect wiih_suggestions',
                'data-url': 'api/issues/search',
            }),
            'authors': forms.SelectMultiple(attrs={
                'class': 'form-control live_multiselect',
                'data-url': 'api/users/search',
            }),
            'finished_at': forms.DateInput(attrs={'class': 'form-control date_picker', }),
        }

    def __init__(self, *args, **kwargs):
        super(PostBaseModelForm, self).__init__(*args, **kwargs)
        self.fields['section'].empty_label = '---'


class PostExtendedModelForm(ModelForm):
    section = forms.ModelChoiceField(
        queryset=Section.objects.filter(is_archived=False),
        label="Рубрика",
        empty_label=None,
        widget=forms.Select(attrs={'class': 'form-control', 'rows': 5})
    )

    class Meta(PostBaseModelForm.Meta):
        model = Post

        fields = ('title', 'description', 'authors', 'kicker', 'xmd', 'section')
        widgets = {
            'title': forms.TextInput(attrs={'class': 'form-control', }),
            'description': forms.HiddenInput(),
            'authors': forms.SelectMultiple(attrs={
                'class': 'form-control live_multiselect',
                'data-url': 'api/users/search',
            }),
            'kicker': forms.TextInput(attrs={'class': 'form-control', }, ),
            'xmd': forms.Textarea(attrs={'class': 'form-control', 'rows': 20, }),
        }

    def __init__(self, *args, **kwargs):
        super(PostExtendedModelForm, self).__init__(*args, **kwargs)
        self.fields['kicker'].required = False
        self.fields['xmd'].required = False
        self.fields['section'].empty_label = None


class WhitelistedPostExtendedModelForm(PostBaseModelForm):
    section = forms.ModelChoiceField(
        queryset=Section.objects.filter(is_archived=False, is_whitelisted=True),
        label="Рубрика",
        empty_label=None,
        widget=forms.Select(attrs={'class': 'form-control', 'rows': 5})
    )


class AdPostExtendedModelForm(PostBaseModelForm):
    issues = forms.ModelChoiceField(
        queryset=Issue.objects.filter(number=0),
        empty_label=None,
        widget=forms.Select(attrs={'class': 'form-control', 'rows': 5})
    )


class CommentModelForm(ModelForm):
    class Meta:
        model = Comment
        fields = ['text', ]
        widgets = {
            'text': forms.Textarea(attrs={'class': 'form-control', 'rows': 5}),
        }



# from django.conf.urls import url
from django.urls import path

from magplan.views import (
    sections,
    stages,
    articles,
    ideas,
    authors,
    index,
    issues,
    posts,
    api,
    preferences,
)

urlpatterns = [

    path('sections/', sections.index, name='sections_index'),
    path('stages/', stages.index, name='stages_index'),

    path('ideas/<int:idea_id>/comments/', ideas.comments, name='ideas_comments'),
    path('ideas/<int:idea_id>/approve/', ideas.approve, name='ideas_approve'),
    path('ideas/<int:idea_id>/vote/', ideas.vote, name='ideas_vote'),
    path('ideas/<int:idea_id>/', ideas.show, name='ideas_show'),
    path('ideas/', ideas.index, name='ideas_index'),

    path('articles/search', articles.search, name='articles_search'),
    path('articles/whitelisted', articles.whitelisted, name='articles_whitelisted'),
    path('articles/advert', articles.advert, name='articles_advert'),
    path('articles/', articles.index, name='articles_index'),

    path('issues/<int:issue_id>/', issues.show, name='issues_show'),
    path('issues/new/', issues.create, name='issues_create'),
    path('issues/', issues.index, name='issues_index'),

    path('posts/<int:post_id>/attachments/delete/', posts.attachment_delete, name='posts_attachment_delete'),
    path('posts/<int:post_id>/comments/', posts.comments, name='posts_comments'),
    path('posts/<int:post_id>/set_stage/', posts.set_stage, name='posts_set_stage'),
    path('posts/<int:post_id>/edit/', posts.edit, name='posts_edit'),
    path('posts/<int:post_id>/edit_meta/', posts.edit_meta, name='posts_edit_meta'),
    path('posts/<int:post_id>/download/', posts.download_content, name='posts_download_content'),
    path('posts/<int:post_id>/send_to_wp/', posts.send_to_wp, name='posts_send_to_wp'),
    path('posts/<int:post_id>/', posts.show, name='posts_show'),
    path('posts', posts.create, name='posts_create'),

    path('api/issues/search/', api.issues_search, name='api_issues_search'),
    path('api/users/search/', api.authors_search, name='api_authors_search'),

    path('authors/new/', authors.new, name='authors_new'),
    path('authors/<int:user_id>/edit/', authors.edit, name='authors_edit'),
    path('authors/<int:user_id>/', authors.show, name='authors_show'),
    path('authors/', authors.index, name='authors_index'),

    path('settings/', preferences.index, name='preferences_index'),



    path('', index.index, name='index_index'),
]



from unittest import TestCase

from tests.plan.tasks.test_utils import get_
from xmd.lexers import PanelBlockLexer


class TestPanelBlockLexer(TestCase):
    def setUp(self):
        self.lexer = PanelBlockLexer()

    def test_basic(self):
        markdown = (
            '[ Panel title\n'
            '\n'
            'Panel content\n'
            '\n'
            ']'
        )

        tokens = self.lexer.parse(markdown)
        token_panel_start = get_(tokens, 'panel_block_default_start', 'type')
        token_panel_end = get_(tokens, 'panel_block_end', 'type')

        if not token_panel_start or not token_panel_end:
            assert False, 'should parse panel block'

        assert token_panel_start['content'] == 'Panel title'

    def test_no_newline_begin_fails(self):
        markdown = (
            '[ Panel title\n'
            'Panel content\n'
            '\n'
            ']'
        )
        tokens = self.lexer.parse(markdown)

        token_panel_start = get_(tokens, 'panel_block_default_start', 'type')
        token_panel_end = get_(tokens, 'panel_block_end', 'type')
        assert not token_panel_start
        assert not token_panel_end

    def test_no_newline_end_fails(self):
        markdown = (
            '[ Panel title\n'
            '\n'
            'Panel content\n'
            ']'
        )
        tokens = self.lexer.parse(markdown)

        token_panel_start = get_(tokens, 'panel_block_default_start', 'type')
        token_panel_end = get_(tokens, 'panel_block_end', 'type')
        assert not token_panel_start
        assert not token_panel_end

    def test_no_newline_both_fails(self):
        markdown = (
            '[ Panel title\n'
            'Panel content\n'

        )
        tokens = self.lexer.parse(markdown)

        token_panel_start = get_(tokens, 'panel_block_default_start', 'type')
        token_panel_end = get_(tokens, 'panel_block_end', 'type')
        assert not token_panel_start
        assert not token_panel_end

    def test_no_title_fails(self):
        markdown = (
            '[ \n'
            'Panel content\n'
            ']'
        )
        tokens = self.lexer.parse(markdown)

        token_panel_start = get_(tokens, 'panel_block_default_start', 'type')
        token_panel_end = get_(tokens, 'panel_block_end', 'type')
        assert not token_panel_start
        assert not token_panel_end

    def test_nested_paragraphs(self):
        markdown = (
            '[ Panel title\n'
            '\n'
            'Panel content 1\n'
            '\n'
            'Panel content 2\n'
            '\n'
            ']'
        )
        tokens = self.lexer.parse(markdown)

        token_panel_start = get_(tokens, 'panel_block_default_start', 'type')
        token_panel_end = get_(tokens, 'panel_block_end', 'type')
        assert token_panel_start
        assert token_panel_end

        paragraph_tokens = [token for token in tokens if token['type'] == 'paragraph']
        assert len(paragraph_tokens) == 2


class TestBlockLexer_lead(TestCase):
    def setUp(self):
        self.lexer = PanelBlockLexer()

    def test_with_token(self):
        markdown: str = (
            '$ Lead\n'
            '\n'
            'Paragraph\n'
        )
        tokens: list = self.lexer.parse(markdown)
        assert len(tokens) == 4
        assert tokens[0]['type'] == 'lead_start'
        assert tokens[1]['type'] == 'paragraph'
        assert tokens[2]['type'] == 'lead_end'
        assert tokens[3]['type'] == 'paragraph'
        
        assert tokens[1]['text'] == 'Lead'
        
    def test_with_token_middle(self):
        markdown: str = (
            '# foo\n'
            '\n'
            '$ Lead\n'
        )
        
        tokens: list = self.lexer.parse(markdown)
        assert len(tokens) == 4
        assert tokens[0]['type'] == 'heading'
        assert tokens[1]['type'] == 'lead_start'
        assert tokens[2]['type'] == 'paragraph'
        assert tokens[3]['type'] == 'lead_end'
        
        assert tokens[2]['text'] == 'Lead'
        
    def test_without_token(self):
        markdown: str = (
            'Lead\n'
            '\n'
            '$ Not a lead\n'
        )
        tokens: list = self.lexer.parse(markdown)
        assert len(tokens) == 4
        assert tokens[0]['type'] == 'lead_start'
        assert tokens[1]['type'] == 'paragraph'
        assert tokens[2]['type'] == 'lead_end'
        assert tokens[3]['type'] == 'paragraph'
        
        assert tokens[1]['text'] == 'Lead'
        
    def test_without_token_middle(self):
        markdown: str = (
            '# foo\n'
            '\n'
            'Lead\n'
        )
        
        tokens: list = self.lexer.parse(markdown)
        assert len(tokens) == 4
        assert tokens[0]['type'] == 'heading'
        assert tokens[1]['type'] == 'lead_start'
        assert tokens[2]['type'] == 'paragraph'
        assert tokens[3]['type'] == 'lead_end'
        
        assert tokens[2]['text'] == 'Lead'
        
class TestBlockLexer_paywall(TestCase):
    def setUp(self):
        self.lexer = PanelBlockLexer()
        
    def test_basic(self):
        markdown: str = (
            '----------\n'
            '\n'
            'Paragraph\n'
        )
        
        tokens: list = self.lexer.parse(markdown)
        assert tokens[0]['type'] == 'paywall'
        
    def test_middle(self):
        markdown: str = (
            'Paragraph\n'
            '\n'
            '----------\n'
            '\n'
            'Paragraph\n'
            
        )
        
        tokens: list = self.lexer.parse(markdown)
        assert tokens[3]['type'] == 'paywall'
        
    def test_cont_paywall(self):
        markdown: str = (
            'Paragraph\n'
            '\n'
            '-----------\n'
            '\n'
            'Paragraph\n'
            
        )
        
        tokens: list = self.lexer.parse(markdown)
        assert tokens[3]['type'] == 'paywall'


#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import sys
from glob import glob

from setuptools import find_packages
from setuptools import setup

sys.path.append(os.path.join(os.path.dirname(__file__), "src"))


# Utility function to read the README file.
# Used for the long_description.  It's nice, because now 1) we have a top level
# README file and 2) it's easier to type in the README file than to put a raw
# string in below ...
def get_path(fname):
    return os.path.join(os.path.dirname(__file__), fname)


install_requirements = [
    "Django>=2.1,<3.2",
]

test_requirements = [
    "django-functest>=1.0.3,<1.1",
    "pytest>=5.3,<5.4",
    "pytest-django",
    "pytest-cov",
    "pytest-pythonpath",
]

test_lint_requirements = [
    "flake8>=3.7,<3.8",
    # Somewhat pin black, such that older code bases can
    # be verified CI without linting them lots
    "black>=20.8b1,<20.9",
    "pre-commit",
]

setup_requirements = [
    "pytest-runner",
]

development_requirements = test_requirements + test_lint_requirements

extras_requirements = {
    "devel": development_requirements,
    "test": test_requirements,
    "testlint": test_lint_requirements,
    "transifex": ["transifex-client"],
}

setup(
    name="magplan",
    version="2.0.6",
    author="Ilya Rusanen",
    author_email="ilya@rusanen.co.uk",
    url="",
    description="Project management system for publishers, magazines and content creators",
    license="MIT",
    keywords=["django", "crm", "markdown"],
    packages=find_packages("src"),
    package_dir={"": "src"},
    py_modules=[
        os.path.splitext(os.path.basename(path))[0] for path in glob("src/*.py")
    ],
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    zip_safe=False,
    install_requires=[
        "amqp==5.0.1; python_version >= '3.6'",
        "asgiref==3.2.10; python_version >= '3.5'",
        "bcrypt==3.2.0; python_version >= '3.6'",
        "billiard==3.6.3.0",
        "boto3==1.16.8",
        "botocore==1.19.8",
        "celery[redis]==5.0.1",
        "certifi==2020.6.20",
        "cffi==1.14.3",
        "chardet==3.0.4",
        "click==7.1.2; python_version >= '2.7' and python_version not in '3.0, 3.1, 3.2, 3.3, 3.4'",
        "click-didyoumean==0.0.3",
        "click-repl==0.1.6",
        "cryptography==3.2.1; python_version >= '2.7' and python_version not in '3.0, 3.1, 3.2, 3.3, 3.4'",
        "django==3.1.2",
        "django-ace==1.0.8",
        "django-admin-list-filter-dropdown==1.0.3",
        "django-dynamic-preferences==1.10.1",
        "django-filter==2.4.0",
        "django-picklefield==3.0.1",
        "django-polymorphic==3.0.0",
        "html2text==2020.1.16",
        "idna==2.10; python_version >= '2.7' and python_version not in '3.0, 3.1, 3.2, 3.3'",
        "jmespath==0.10.0; python_version >= '2.6' and python_version not in '3.0, 3.1, 3.2, 3.3'",
        "kombu==5.0.2; python_version >= '3.6'",
        "mistune==0.8.4",
        "paramiko==2.7.2",
        "persisting-theory==0.2.1",
        "prompt-toolkit==3.0.8; python_full_version >= '3.6.1'",
        "psycopg2==2.8.6",
        "pycparser==2.20; python_version >= '2.7' and python_version not in '3.0, 3.1, 3.2, 3.3'",
        "pymysql==0.10.1",
        "pynacl==1.4.0; python_version >= '2.7' and python_version not in '3.0, 3.1, 3.2, 3.3'",
        "python-dateutil==2.8.1; python_version >= '2.7' and python_version not in '3.0, 3.1, 3.2, 3.3'",
        "python-slugify==4.0.1",
        "pytz==2020.1",
        "redis==3.5.3",
        "requests==2.24.0",
        "s3transfer==0.3.3",
        "six==1.15.0; python_version >= '2.7' and python_version not in '3.0, 3.1, 3.2, 3.3'",
        "sqlparse==0.4.1; python_version >= '3.5'",
        "sshtunnel==0.2.1",
        "text-unidecode==1.3",
        "urllib3==1.25.11; python_version != '3.4'",
        "vine==5.0.0; python_version >= '3.6'",
        "wcwidth==0.2.5",
    ],
    dependency_links=[],
    extra_require={},
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Environment :: Web Environment",
        "Framework :: Django",
        "Intended Audience :: Developers",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3 :: Only",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: Implementation :: CPython",
        "Programming Language :: Python :: Implementation :: PyPy",
        "Topic :: Internet :: WWW/HTTP :: Dynamic Content",
        "Topic :: Software Development",
        "Topic :: Software Development :: Libraries :: Application Frameworks",
    ],
    include_package_data=True,
    setup_requires=setup_requirements,
    tests_require=test_requirements,
    extras_require={},
)



import os
from contextlib import contextmanager

import boto3


@contextmanager
def S3Client():
    S3_ACCESS_KEY = os.environ.get('S3_ACCESS_KEY')
    S3_SECRET_KEY = os.environ.get('S3_SECRET_KEY')
    S3_BUCKET_NAME = os.environ.get('S3_BUCKET_NAME')

    if not all((S3_ACCESS_KEY, S3_SECRET_KEY)):
        raise ValueError('Either S3_ACCESS_KEY or S3_SECRET_KEY not provided')

    client = boto3.resource(
        's3',
        aws_access_key_id=S3_ACCESS_KEY,
        aws_secret_access_key=S3_SECRET_KEY,
    )
    yield (client, S3_BUCKET_NAME)



import re

from mistune import BlockLexer


class PanelBlockLexer(BlockLexer):
    def __init__(self, *args, **kwargs):
        super(PanelBlockLexer, self).__init__(*args, **kwargs)
        self.has_lead = False

        self.enable_panel()
        self.enable_paywall()

    def enable_panel(self):
        self.rules.panel_block = re.compile(r'\[ (.+)\n'
                                            r'\n'
                                            r'((?:.*?\n)*?)'
                                            r'\n'
                                            r'\]'
                                            )
        self.default_rules.insert(3, 'panel_block')
        
    def enable_paywall(self):
        self.rules.paywall_block = re.compile(
            r'^-{10,}'
        )
        self.default_rules.insert(1, 'paywall_block')

    def parse_panel_block(self, m):
        title = m.group(1)
        content = m.group(2)

        # We don't want trailing \n
        title = title.strip()
        content = content.strip()

        # Select style for panel,
        # or failback to default
        if title == 'WWW':
            self.tokens.append({
                'type': 'panel_block_www_start'
            })
        elif title == 'INFO':
            self.tokens.append({
                'type': 'panel_block_info_start'
            })
        elif title == 'WARNING':
            self.tokens.append({
                'type': 'panel_block_warning_start'
            })
        elif title == 'GREETING':
            self.tokens.append({
                'type': 'panel_block_greeting_start'
            })
        elif title == 'TERM':
            self.tokens.append({
                'type': 'panel_block_term_start'
            })
        else:
            self.tokens.append({
                'type': 'panel_block_default_start',
                'content': title
            })

        # Add nested tokens
        if title == 'TERM':
            # We treat term content as single
            # code block without nested elements
            self.tokens.append({
                'type': 'panel_block_term_code',
                'content': content
            })

        else:
            self.parse(content)

        # Add panel end token
        self.tokens.append({
            'type': 'panel_block_end'
        })

    def parse_paragraph(self, m) -> None:
        """Parse paragraph, including lead case

        Lead is a first paragraph in document.
        If 

        :param m: Match group for paragraph, where m.group(1) is text.
        :return:
        """
        text: str = m.group(1).rstrip('\n')
        is_lead: bool = False # Local marker
        
        # Are we parsing lead?
        if not self.has_lead:
            is_lead = True
            self.has_lead = True
        
        # If it's lead, wrap with lead tokens,
        # otherwise parse as ususal paragraph
        if is_lead:
            self.tokens.append({'type': 'lead_start'})
            
            # Strip '$ ' from text if exists
            if text.startswith('$ '):
                text = text[2:]
            self.has_lead = True

        self.tokens.append({'type': 'paragraph', 'text': text})

        if is_lead:
            self.tokens.append({'type': 'lead_end'})

        
    def parse_paywall_block(self, m):
        self.tokens.append({'type': 'paywall'})



from django.test import Client, TestCase
from plan.tests import _User, _Sections, _Section, _Stages, _Post, _Postype, _Issue
from django.urls import reverse
from datetime import datetime
from main.models import Issue


class TestCreate(TestCase):
    @classmethod
    def setUpTestData(cls):
        cls.user = _User()

    def setUp(self):
        self.ROUTE_NAME = 'issues_create'
        self.client = Client()

    def test_redirect_to_login_if_not_authenticated(self):
        response = self.client.get(reverse(self.ROUTE_NAME))
        self.assertRedirects(response, '%s?next=/admin/issues/new/' % reverse('login'))

    def test_should_render_issue_form(self):
        self.client.force_login(user=self.user)
        response = self.client.get(reverse(self.ROUTE_NAME))

        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, 'magplan/issues/_form.html')

    def test_issue_should_be_created(self):
        self.client.force_login(user=self.user)

        published_at = datetime.now()
        response = self.client.post(reverse(self.ROUTE_NAME), {
            'title': 'title',
            'description': 'description',
            'number': '123',
            'published_at': published_at.strftime('%d.%m.%Y'),
        })

        self.assertRedirects(response, reverse('issues_index'))
        self.assertEqual(Issue.objects.filter(number=123), 1)



"""
Every block element test will be automatically
wrapped inside `<p></p>\n`. Thats why every block

test should include this wrapper tag.
"""
from io import BytesIO
from unittest import TestCase
from unittest.mock import patch, Mock

import pytest
from django.core.files import File
from django_dynamic_fixture import G

from magplan.models import Attachment
from magplan.xmd.renderer import XMDRenderer
from magplan.xmd.mappers import plan_internal_mapper


@pytest.mark.django_db
class TestImage(TestCase):
    MOCK_SRC = 'dummy.jpg'
    MOCK_TITLE = 'title'
    MOCK_ALT_TEXT = 'alt_text'

    def setUp(self):
        file1 = File(name='file1.jpg', file=BytesIO(b'abcdef'))
        attachment1 = G(Attachment, original_filename='user_friendly_filename1.jpg', file=file1)

        self.mock_image_mapper = Mock()

        self.renderer = XMDRenderer(image_mapper=self.mock_image_mapper, attachments=[attachment1])

        self.expected_html = (
            '<figure>'
            '<img conf="dummy.jpg" alt="alt_text" /><figcaption>alt_text</figcaption>'
            '</figure>'
        )

    def test_render_image(self):
        self.mock_image_mapper.return_value = self.MOCK_SRC

        html = self.renderer.image(self.MOCK_SRC, self.MOCK_TITLE, self.MOCK_ALT_TEXT)

        self.mock_image_mapper.assert_called_with(self.MOCK_SRC, self.renderer.attachments)
        assert html == self.expected_html



import datetime
from typing import Generator, Tuple

from django.contrib.auth.decorators import login_required
from django.db.models import Q
from django.shortcuts import render

from magplan.models import Idea, Issue, Post


def get_schedule_starttime() -> Tuple[datetime.datetime, datetime.datetime]:
    """Get time range for schedule

    Counts two dates:
    * beginning of this week
    * end of next week
    :return:
    """
    today = datetime.datetime.today().replace(hour=0, minute=0, second=0)
    this_week_begin = today - datetime.timedelta(days=today.weekday())
    next_week_end = this_week_begin + datetime.timedelta(days=14)

    return this_week_begin, next_week_end


def daterange(
    start_date: datetime.datetime,
    end_date: datetime.datetime,
    skip_weekend: bool = True,
) -> Generator:
    """Yield every day date between two dates.

    Pre-fixes provided dates to include full day cycle.
    @generator
    """
    weekend_days = (5, 6)

    # Fix daterange to include full day cycle to prevent comparison failures
    start_date = start_date.replace(hour=0, minute=0, second=0)
    end_date = end_date.replace(hour=23, minute=59, second=59)
    start_day_weekday = start_date.weekday()

    for days in range(int((end_date - start_date).days)):
        if skip_weekend is False:
            yield start_date + datetime.timedelta(days)
            continue

        next_date_weekday = (start_day_weekday + days) % 7
        if next_date_weekday not in weekend_days:
            yield start_date + datetime.timedelta(days)


# Create your views here.
@login_required
def index(request):
    self_posts = (
        Post.objects.prefetch_related('stage')
        .filter(
            Q(stage__assignee__isnull=False, stage__assignee=request.user)
            | Q(stage__assignee__isnull=True, editor=request.user)
        )
        .exclude(stage__slug__in=['vault', 'published'])
    )

    need_to_vote = Idea.objects.filter(approved__isnull=True).exclude(
        votes__user=request.user
    )

    opened_issues = Issue.objects.filter(
        posts__stage__slug__in=[
            'waiting',
            'proofreading_editor',
            'precheck',
            'spellcheck',
            'markup',
            'proofreading_spell',
            'proofreading_chief_dpt',
            'proofreading_chief',
            'publishing',
        ]
    ).distinct()

    # Count schedule
    schedule = {}
    time_start, time_end = get_schedule_starttime()
    scheduled_posts = Post.objects.filter(
        published_at__gte=time_start, published_at__lte=time_end
    )

    for day in daterange(time_start, time_end):
        schedule[day] = list(
            filter(
                lambda p: day.astimezone()
                <= p.published_at
                <= day.astimezone() + datetime.timedelta(days=1),
                scheduled_posts,
            )
        )

    return render(
        request,
        'magplan/index/index.html',
        {
            'self_posts': self_posts,
            'need_to_vote': need_to_vote,
            'opened_issues': opened_issues,
            'schedule': schedule,
            'today': datetime.datetime.today().replace(hour=0, minute=0, second=0),
        },
    )



import datetime
import os
from typing import List, Tuple, Optional

import django_filters
import html2text
from magplan.conf import settings as config
from django.contrib import messages
from django.contrib.auth.decorators import login_required
from django.core.mail import EmailMultiAlternatives
from django.db.models import Count
from django.shortcuts import redirect, render
from django.template.loader import render_to_string

from magplan.models import Idea, Issue, Vote
from magplan.forms import CommentModelForm, IdeaModelForm, PostBaseModelForm, IDEA_AUTHOR_SELF_CHOICE
from magplan.tasks.send_idea_comment_notification import send_idea_comment_notification
from magplan.tasks.send_idea_notification import send_idea_notification
from magplan.utils import safe_cast

IDEAS_PER_PAGE = 20

# HACK
AUTHOR_TYPE_DAY = datetime.date(2019, 6, 29)


class IdeaApprovedFilter(django_filters.FilterSet):
    approved = django_filters.BooleanFilter(field_name='approved')

    class Meta:
        model = Idea
        fields = ['approved', ]


@login_required
def index(request):
    if request.method == 'POST':
        form = IdeaModelForm(request.POST)
        if form.is_valid():
            idea = form.save(commit=False)
            idea.editor = request.user.user
            idea.save()

            # Save authors, if existing specified
            if idea.author_type == Idea.AUTHOR_TYPE_EXISTING:
                form.save_m2m()

                # Handle virtual SELF choice (which is actual AUTHOR_TYPE_EXISTING)
                # We did it after actual m2m save as form.save_m2m()will overwrite any
                # model assignments usgin empty Form authors field. So saving one more time
                if request.POST.get('author_type') == IDEA_AUTHOR_SELF_CHOICE.slug:
                    idea.authors.add(request.user.user)
                    idea.save()

            send_idea_notification.delay(idea.id)

            # Clear idea form to prevent rendering pre-filled form
            form = IdeaModelForm()

            messages.add_message(
                request, messages.SUCCESS,
                'Идея «%s» успешно выдвинута на голосование!' % idea.title)

            return redirect('ideas_show', idea_id=idea.id)

    form = IdeaModelForm()

    ideas = (Idea.objects
             .annotate(voted=Count("votes"))
             .prefetch_related('editor', )
             .order_by('-created_at'))

    # filters
    filter_ = request.GET.get('filter', None)
    if filter_ == 'voted':
        ideas = ideas.filter(approved=None)
    elif filter_ == 'self':
        ideas = ideas.filter(editor=request.user.user)
    elif filter_ == 'approved':
        ideas = ideas.filter(approved=True)
    elif filter_ == 'rejected':
        ideas = ideas.filter(approved=False)
    elif filter_ == 'no_author':
        ideas = ideas.filter(
            author_type=Idea.AUTHOR_TYPE_NO,
            created_at__gte=AUTHOR_TYPE_DAY,  # HACK
        )

    ideas = ideas.all()

    return render(request, 'magplan/ideas/index.html', {
        'ideas': ideas,
        'form': form,
        'filter_': filter_,
    })


def _get_suggestion_issues() -> Tuple[Optional[Issue], List[Issue]]:
    """Returns issues suggesion for "new Post" form.

    Retrieves five last opened issues, where last in array is newest
    """
    article_stages_for_opened_issues = (
        'waiting',
        'proofreading_editor',
        'precheck',
        'spellcheck',
        'markup',
        'proofreading_spell',
        'proofreading_chief_dpt',
        'proofreading_chief',
        'publishing',
    )
    opened_issues = \
        Issue.objects \
            .filter(posts__stage__slug__in=article_stages_for_opened_issues) \
            .order_by('number') \
            .distinct() \
            .all()[:5]

    if not opened_issues:
        return None, []

    # Convert QuerySet to ordinary list to split for initial and least
    opened_issues: List[Issue] = list(opened_issues)
    initial_issue = opened_issues.pop()
    opened_issues.reverse()

    return initial_issue, opened_issues


@login_required
def show(request, idea_id):
    idea = Idea.objects.prefetch_related('votes__user').get(id=idea_id)
    initial_issues_suggesion, issues_suggesions = _get_suggestion_issues()
    form = PostBaseModelForm(initial={
        'issues': initial_issues_suggesion,
        'finished_at': datetime.datetime.now() + datetime.timedelta(days=3),
    }, instance=idea)

    return render(request, 'magplan/ideas/show.html', {
        'idea': idea,
        'form': form,
        'issues_suggesions': issues_suggesions,
        'comment_form': CommentModelForm(),
        'AUTHOR_TYPE_CHOICES': Idea.AUTHOR_TYPE_CHOICES,
    })


@login_required
def vote(request, idea_id):
    DEFAULT_VOTE_SCORE = Vote.SCORE_50

    idea = Idea.objects.prefetch_related('votes__user').get(id=idea_id)
    vote = Vote(idea=idea, user=request.user.user)
    score: int

    if request.method == 'POST':
        score = request.POST.get('score', DEFAULT_VOTE_SCORE)
    else:
        score = request.GET.get('score', DEFAULT_VOTE_SCORE)

    score = safe_cast(score, to=int, on_error=DEFAULT_VOTE_SCORE)

    allowed_scored: List[int] = [
        score_choice[0] for score_choice in Vote.SCORE_CHOICES
    ]

    if score not in allowed_scored:
        score = Vote.SCORE_50

    vote.score = score
    vote.save()

    messages.add_message(request, messages.SUCCESS, 'Ваш голос учтен. Спасибо!')

    return redirect('ideas_show', idea_id=idea.id)


@login_required
def approve(request, idea_id):
    idea = Idea.objects.prefetch_related('votes__user').get(id=idea_id)

    if request.method == 'POST':
        idea.approved = (True if request.POST.get(
            'approve', False) == '1' else False)
        idea.save()
        messages.add_message(request, messages.INFO, 'Статус идеи изменен.')

        # send email
        if idea.approved is True and idea.editor != request.user.user:
            subject = f'Идея «{idea}» прошла голосование! Ждем статью'
            html_content = render_to_string('email/idea_approved.html', {
                'idea': idea,
                'APP_URL': os.environ.get('APP_URL', None),
            })
            text_content = html2text.html2text(html_content)
            msg = EmailMultiAlternatives(
                subject, text_content, config.PLAN_EMAIL_FROM, [idea.editor.email])
            msg.attach_alternative(html_content, "text/html")
            msg.send()

    return redirect('ideas_show', idea_id=idea.id)


@login_required
def comments(request, idea_id):
    idea = Idea.objects.prefetch_related('votes__user').get(id=idea_id)

    if request.method == 'POST':
        comment_form = CommentModelForm(request.POST)

        comment = comment_form.save(commit=False)
        comment.commentable = idea
        comment.user = request.user.user

        if comment_form.is_valid():
            comment_form.save()

            send_idea_comment_notification.delay(comment.id)

            return redirect('ideas_show', idea.id)
    else:
        return redirect('ideas_show', idea.id)



import datetime
import io
import os
from typing import List
from zipfile import ZIP_DEFLATED, ZipFile

import html2text
from django.conf import settings

from magplan.conf import settings as config
from django.contrib import messages
from django.contrib.auth.decorators import login_required
from django.core.mail import EmailMultiAlternatives
from django.db import transaction
from django.http import HttpRequest, HttpResponseForbidden
from django.shortcuts import HttpResponse, redirect, render
from django.shortcuts import get_object_or_404
from django.template import Context, Template
from django.template.loader import render_to_string
from slugify import slugify

from magplan.models import (
    Attachment,
    Comment,
    Idea,
    Post,
    Stage,
    User,
)
from magplan.forms import (
    CommentModelForm,
    PostBaseModelForm,
    PostExtendedModelForm,
    PostMetaForm,
)
from magplan.tasks.send_post_comment_notification import send_post_comment_notification
from magplan.tasks.upload_post_to_wp import upload_post_to_wp


def _get_arbitrary_chunk(post: Post) -> str:
    """Render instance specific template code

    Used to render some arbitrary HTML code in a context of Post instance.
    Useful to provide sensitive HTML template, which can't be committed
    into Git repository directly or may vary for each particular instance.

    :param post: Post instance to use in template
    :return: Rendered template string
    """
    instance_template = Template(config.PLAN_POSTS_INSTANCE_CHUNK)
    instance_chunk = instance_template.render(Context({"post": post}))
    return instance_chunk


def _create_system_comment(
        action_type, user, post, changelog=None, attachments=None, stage=None
) -> Comment:
    """Create auto-generated system comment with post changes logs
    
    :param action_type:
    :param user:
    :param post:
    :param changelog:
    :return:
    """
    if not config.SYSTEM_USER_ID:
        return None

    if not attachments:
        attachments = ()
    if not stage:
        stage = post.stage

    system_user = User.objects.get(id=config.SYSTEM_USER_ID)
    comment = Comment()
    comment.commentable = post
    comment.type = Comment.TYPE_SYSTEM
    comment.user = system_user

    # Depending on action type, fill different fields
    meta = {
        "action": action_type,
        "user": {"id": user.id, "str": user.__str__()},
        "files": [],
    }

    if action_type == Comment.SYSTEM_ACTION_CHANGE_META:
        meta["changelog"] = changelog

    elif action_type == Comment.SYSTEM_ACTION_UPDATE:
        if len(attachments) > 0:
            meta["files"] = [
                {"id": a.id, "str": a.original_filename} for a in attachments
            ]

    elif action_type == Comment.SYSTEM_ACTION_SET_STAGE:
        meta["stage"] = {"id": post.stage.id, "str": post.stage.__str__()}

    # Assign builded meta to comment and save
    comment.meta["comment"] = meta
    comment.save()

    return comment


def _generate_changelog_for_form(form: PostMetaForm) -> List[str]:
    """Iterate over all changed attributes and stage changes in logs

    :param form: Django form
    :return: list, where each element is changelog line
    """
    changelog = []
    changed_fields = form.changed_data.copy()
    if "wp_id" in changed_fields:
        changed_fields.remove("wp_id")

    __ = lambda form, field: (
        ", ".join([str(i) for i in form.initial.get(field)]),
        ", ".join([str(i) for i in form.cleaned_data.get(field)]),
    )
    _ = lambda form, field: (form.initial.get(field), form.cleaned_data.get(field))
    for changed_field in changed_fields:
        log = None

        if changed_field == "issues":
            log = '* выпуски сменились с "{0}" на "{1}"'.format(
                *__(form, changed_field)
            )
        elif changed_field == "editor":
            # Initial ForeignKey value is stored as int. Populate it
            args = _(form, changed_field)
            init_editor = str(User.objects.get(id=args[0]))
            new_args = (init_editor, args[1])
            log = '* редактор cменился с "{0}" на "{1}"'.format(*new_args)
        elif changed_field == "finished_at":
            log = '* дедлайн этапа cменился с "{0}" на "{1}"'.format(
                *_(form, changed_field)
            )
        elif changed_field == "published_at":
            log = '* дата публикации сменилась с "{0}" на "{1}"'.format(
                *_(form, changed_field)
            )

        if log:
            changelog.append(log)

    return changelog


def _authorize_stage_change(user: User, post: Post, new_stage_id: int) -> bool:
    """Check if user is authorized to set stage for post

    :param user: User instance
    :param post: Post instance
    :param new_stage_id: Stage to to set for post
    :return: True if authorized, otherwise False
    """
    legit_stages = (post.stage.prev_stage_id, post.stage.next_stage_id)
    
    if new_stage_id in legit_stages and post.assignee == user:
        return True

    if user.has_perm("main.edit_extended_post_attrs"):
        return True

    return False


def _save_attachments(files: List, post: Post, user: User) -> List[Attachment]:
    attachments = []

    with transaction.atomic():
        for file in files:
            # Delete files with the same filename,
            # uploaded for current post. Emulates overwrite without
            # custom FileSystemStorage
            Attachment.objects.filter(
                post=post, original_filename=file.name
            ).delete()

            attachment = Attachment(
                post=post, user=user, original_filename=file.name
            )  # save original filename

            # Slugify original filename and save with safe one
            filename, extension = os.path.splitext(file.name)
            file.name = "%s%s" % (slugify(filename), extension)

            # Assign file object with slugified filename as name,
            # original is copied by value
            attachment.file = file

            if file.content_type in ("image/png", "image/jpeg"):
                attachment.type = Attachment.TYPE_IMAGE
            elif file.content_type == "application/pdf":
                attachment.type = Attachment.TYPE_PDF
            else:
                attachment.type = Attachment.TYPE_FILE

            attachment.save()
            attachments.append(attachment)

    return attachments


@login_required
def show(request, post_id):
    post = Post.objects.prefetch_related(
        "editor", "authors", "stage", "section", "issues", "comments__user"
    ).get(id=post_id)

    post_meta_form = PostMetaForm(
        initial={'wp_id': post.meta.get('wpid')}, instance=post
    )

    return render(
        request,
        "magplan/posts/show.html",
        {
            "post": post,
            "stages": Stage.objects.order_by("sort").all(),
            "instance_chunk": _get_arbitrary_chunk(post),
            "comment_form": CommentModelForm(),
            "meta_form": post_meta_form,
            "TYPE_CHOICES": Comment.TYPE_CHOICES,
            "SYSTEM_ACTION_CHOICES": Comment.SYSTEM_ACTION_CHOICES,
        },
    )


@login_required
def create(request):
    if request.method == "POST":
        form = PostBaseModelForm(request.POST)

        if form.is_valid():
            post = form.save(commit=False)
            post.editor = request.user.user
            post.stage = Stage.objects.get(slug="waiting")

            post.save()
            form.save_m2m()

            idea = Idea.objects.get(id=request.POST.get("idea_id", None))
            idea.post = post
            idea.save()

            return redirect("posts_show", post.id)

    else:
        return HttpResponse(status=405)


@login_required
def edit(request, post_id):
    post = Post.objects.prefetch_related(
        "editor", "authors", "stage", "section", "issues"
    ).get(id=post_id)

    if request.method == "POST":
        form = PostExtendedModelForm(request.POST, request.FILES, instance=post)

        files = request.FILES.getlist("attachments")
        attachments = _save_attachments(files, post, request.user.user)

        if form.is_valid():
            post.imprint_updater(request.user.user)
            form.save()

            _create_system_comment(
                Comment.SYSTEM_ACTION_UPDATE,
                request.user.user,
                post,
                attachments=attachments,
            )
            messages.add_message(
                request, messages.SUCCESS, "Пост «%s» успешно отредактирован" % post
            )

            return redirect("posts_edit", post_id)

        else:
            messages.add_message(
                request, messages.ERROR, "При обновлении поста произошла ошибка ввода"
            )

    else:
        form = PostExtendedModelForm(instance=post)

    return render(request, "magplan/posts/edit.html", {"post": post, "form": form})


@login_required
def edit_meta(request, post_id):
    def _get_value_repr(value) -> str:
        try:
            iter(value)
            return ", ".join(list(map(str(o) for o in value)))
        except TypeError as te:
            return str(value)

    try:
        post = Post.objects.get(id=post_id)
    except Post.DoesNotExist:
        return HttpResponse(status=401)

    if request.method == "POST":
        form = PostMetaForm(request.POST, instance=post)
        if not form.is_valid():
            return HttpResponse(status=403)

        # Manually set new Wordpress ID as it's ignored by form
        form.instance.meta["wpid"] = form.cleaned_data.get("wp_id")

        published_at = form.cleaned_data.get("published_at")
        if published_at:
            dt = published_at.replace(hour=10, minute=0, second=0)
            form.instance.published_at = dt

        form.save()

        # Create system comment with changelog
        changelog = _generate_changelog_for_form(form)
        if len(changelog) > 0:
            _create_system_comment(
                Comment.SYSTEM_ACTION_CHANGE_META, request.user.user, post, changelog
            )

        messages.add_message(request, messages.INFO, f"Пост {post} успешно обновлен!")

    return redirect("posts_show", post_id)


@login_required
def set_stage(request, post_id, system=Comment.TYPE_SYSTEM):
    post = Post.objects.prefetch_related("stage__n_stage", "stage__p_stage").get(
        id=post_id
    )

    if request.method == "POST" and request.POST.get("new_stage_id"):
        if not _authorize_stage_change(
                request.user.user, post, int(request.POST.get("new_stage_id"))
        ):
            return HttpResponseForbidden()

        stage = Stage.objects.get(id=request.POST.get("new_stage_id", None))

        # set deadline to current stage durtion. If no duration, append 1 day
        duration = stage.duration if stage.duration else 1
        post.finished_at = post.finished_at + +datetime.timedelta(days=duration)
        post.stage = stage
        post.imprint_updater(request.user.user)
        post.save()
        messages.add_message(
            request, messages.INFO, "Текущий этап статьи «%s» обновлен" % post
        )

        # Create system comment
        _create_system_comment(
            Comment.SYSTEM_ACTION_SET_STAGE, request.user.user, post, stage=post.stage
        )

        # TODO: extract method
        # Send email if stage allows it
        if post.assignee != request.user.user and stage.skip_notification is False:
            subject = f"На вас назначена статья «{post}»"
            html_content = render_to_string(
                "email/assigned_to_you.html",
                {"post": post, "APP_URL": os.environ.get("APP_URL", None)},
            )
            text_content = html2text.html2text(html_content)
            msg = EmailMultiAlternatives(
                subject, text_content, config.PLAN_EMAIL_FROM, [post.assignee.email]
            )
            msg.attach_alternative(html_content, "text/html")
            msg.send()

            return redirect("posts_show", post_id)

    return redirect("posts_show", post.id)


@login_required
def comments(request, post_id):
    post = Post.objects.prefetch_related(
        "editor", "authors", "stage", "section", "issues", "comments__user"
    ).get(id=post_id)

    if request.method == "POST":
        form = CommentModelForm(request.POST)

        comment = form.save(commit=False)
        comment.commentable = post
        comment.user = request.user.user

        if form.is_valid():
            form.save()

            # Send notification to users with 'recieve_post_email_updates' permission
            send_post_comment_notification.delay(comment.id)
            return redirect("posts_show", post_id)

    return render(
        request,
        "magplan/posts/show.html",
        {
            "post": post,
            "stages": Stage.objects.order_by("sort").all(),
            "form": CommentModelForm(),
        },
    )


@login_required
def attachment_delete(request, post_id):
    if request.method == "POST":
        try:
            attachemnt = Attachment.objects.get(
                id=request.POST.get("attachment_id", None), post_id=post_id
            )
            attachemnt.delete()

        except:
            # TODO: fix too broad exception
            pass

        return HttpResponse(status=204)


@login_required
def download_content(request: HttpRequest, post_id: int) -> HttpResponse:
    """Get all files of requested type for post_id and stream to client as ZIP archive

    :param request: Django request object
    :param post_id: post_id
    :return: Django HttpResponse with file
    """
    if request.method == "GET":
        s = io.BytesIO()
        zipfile = ZipFile(s, "w", ZIP_DEFLATED)
        attachments = Attachment.objects.filter(
            post_id=post_id, type=Attachment.TYPE_IMAGE
        ).all()

        for attachment in attachments:
            fs_path = "%s/%s" % (settings.MEDIA_ROOT, attachment.file.name)
            filename = attachment.original_filename
            try:
                zipfile.write(fs_path, arcname=filename)
            except Exception as e:
                # TODO: handle not found files
                pass
        zipfile.close()

        resp = HttpResponse(s.getvalue(), content_type="application/x-zip-compressed")
        resp["Content-Disposition"] = f"attachment; filename=content_{post_id}.zip"
        return resp


def send_to_wp(request: HttpRequest, post_id: int) -> HttpResponse:
    if request.method == "GET":
        post = get_object_or_404(Post, id=post_id)
        upload_post_to_wp.delay(post.id)

        messages.add_message(
            request, messages.INFO, "Пост «%s» отправлен в Wordpress" % post
        )

        return redirect("posts_show", post.id)



import io
from unittest import TestCase

import pytest
from django.core.files import File
from django_dynamic_fixture import G

from magplan.models import Attachment
from magplan.xmd.utils import get_attachment_original_filename


@pytest.mark.django_db
class Test_get_attachment_original_filename(TestCase):
    def setUp(self):
        file1 = File(
            name='file1.jpg',
            file=io.BytesIO(b'abcdef')
        )
        file2 = File(
            name='file2.jpg',
            file=io.BytesIO(b'abcdef')
        )

        attachment1 = G(Attachment, original_filename='user_friendly_filename1.jpg', file=file1)
        attachment2 = G(Attachment, original_filename='user_friendly_filename2.jpg', file=file2)
        self.attachments = [attachment1, attachment2]

    def test_retrieve_filename(self):
        # filename is returned with full URL path and internal hash
        # So e're checking only filename matches.
        filename = get_attachment_original_filename('user_friendly_filename1.jpg', self.attachments)
        assert 'file1' in filename

    def test_non_existing_filename(self):
        filename = get_attachment_original_filename('404.jpg', self.attachments)
        assert not filename
